# frozen_string_literal: true

module Api
  module Admin
    class StatisticsController < BaseController
      before_action :set_calendar_month

      def inspections # rubocop:disable Metrics/MethodLength, Metrics/AbcSize
        authorize :statistic, :inspections?
        # [{
        #      day: 1,
        #      planned: 100,
        #      reviewed: 90,
        #      canceled: 10
        #  },
        #  {
        #      day: 2,
        #      planned: 100,
        #      reviewed: 90,
        #      canceled: 10
        #  }
        #  ... (все для данного месяца)
        # ]
        @planned = @calendar_month.inspections.planned.group_by_day(:date, format: '%e').count
        @started = @calendar_month.inspections.started.group_by_day(:date, format: '%e').count
        @canceled = @calendar_month.inspections.canceled.group_by_day(:date, format: '%e').count
        @finished = @calendar_month.inspections.finished.group_by_day(:date, format: '%e').count
        @remediation_planned = @calendar_month.inspections.remediation_planned.group_by_day(:date, format: '%e').count

        total_days = Time.days_in_month(@calendar_month.date.month, @calendar_month.date.year)
        @result = []

        1.upto(total_days) do |i|
          @result << {
            day: i,
            planned: @planned[i.to_s].presence || 0,
            started: @started[i.to_s].presence || 0,
            canceled: @canceled[i.to_s].presence || 0,
            finished: @finished[i.to_s].presence || 0,
            remediation_planned: @remediation_planned[i.to_s].presence || 0
          }
        end

        render json: @result
      end

      def inspectors # rubocop:disable Metrics/MethodLength, Metrics/AbcSize
        authorize :statistic, :inspectors?
        # [{
        #      day: 1,
        #      total: 100,
        #      на линии: 90,
        #      отдых: 10,
        #      невыход: 0
        #  },
        #  {
        #      day: 2,
        #      total: 100,
        #      на линии: 90,
        #      отдых: 10,
        #      невыход: 0
        #  }
        #  ... (все для данного месяца)
        # ]

        @result = []
        # всего (общее кол-во специалистов в этот день)
        @total = @calendar_month.calendar_days.group_by_day(:date, format: '%e').count

        # на линии (специалисты с назначенными проверками)
        @working = @calendar_month
                   .calendar_days
                   .joins(:inspections)
                   .where(
                     extended_kind: nil,
                     kind: CalendarDay::KINDS[:working],
                     inspections: { state: %i[planned started finished] }
                   )
                   .distinct.group_by_day(:date, format: '%e')
                   .count

        # невыход (больничный, отпуск за свой счет)
        # туду доп отпуск?
        @not_working_by_reason = @calendar_month
                                 .calendar_days
                                 .where(extended_kind: [CalendarDay::EXTENDED_KINDS[:sick], CalendarDay::EXTENDED_KINDS[:non_paid_vacation]])
                                 .group_by_day(:date, format: '%e')
                                 .count

        # отдых (специалисты со статусами: выходной, отпуск - запланированный, доп. день отпуска)
        @not_working = @calendar_month
                       .calendar_days
                       .where(
                         kind: [CalendarDay::KINDS[:vacation], CalendarDay::KINDS[:user_non_working]],
                         extended_kind: [nil, CalendarDay::EXTENDED_KINDS[:extra_vacation]]
                       )
                       .group_by_day(:date, format: '%e')
                       .count

        # свободен (у специалиста раб. день, но не назначены проверки)
        @without_inspections = @calendar_month
                               .calendar_days
                               .includes(:inspections, :user)
                               .where(
                                 kind: CalendarDay::KINDS[:working],
                                 inspections: { id: nil },
                                 users: { project_type: current_project_type }
                               )
                               .group_by_day(:date, format: '%e')
                               .count

        total_days = Time.days_in_month(@calendar_month.date.month, @calendar_month.date.year)
        @result = []

        1.upto(total_days) do |i|
          @result << {
            day: i,
            total: @total[i.to_s].presence || 0,
            working: @working[i.to_s].presence || 0,
            not_working_by_reason: @not_working_by_reason[i.to_s].presence || 0,
            not_working: @not_working[i.to_s].presence || 0,
            without_inspections: @without_inspections[i.to_s].presence || 0
          }
        end

        render json: @result
      end

      def working_inspectors
        authorize :statistic, :working_inspectors?

        searched_user_ids = UserSearcher.call(params, current_user, current_project_type).pluck(:id)
        user_ids = @calendar_month.calendar_days.joins(:inspections).distinct
                                  .where(
                                    inspections: { date: Date.current, state: %i[planned started finished] },
                                    user_id: searched_user_ids
                                  )
                                  .pluck(:user_id)

        @users = User.where(id: user_ids)
      end

      private

      def set_calendar_month
        @date = begin
                  Date.strptime(params[:month], '%Y-%m')
                rescue StandardError
                  raise ActiveRecord::RecordNotFound
                end

        @calendar_month = CalendarMonth.find_or_initialize_by(date: @date, project_type: current_project_type)
        @calendar_month.save! if @calendar_month.new_record? && action_name != 'show'
      end
    end
  end
end
