# frozen_string_literal: true

module Api
  module Admin
    class UsersController < BaseController
      before_action :set_user, only: %i[show update destroy reactivate]
      before_action :set_users, only: %i[mass_update]

      def index
        @users = UserSearcher.call(params, current_user, current_project_type)
        @pagy = Pagy.new_from_searchkick(@users)
        authorize :user, :index?
      end

      def show
        authorize @user
      end

      def create
        authorize :user, :create?

        result = Users::Create.call(user_params, current_user)

        if result.success?
          @user = result.value!
          render :show, status: :created
        else
          render json: { code: 422, errors: result.failure }, status: :unprocessable_entity
        end
      end

      def update
        authorize @user

        result = Users::Update.call(@user, user_params, current_user)

        if result.success?
          render :show
        else
          render json: { code: 422, errors: @user.errors }, status: :unprocessable_entity
        end
      end

      def destroy
        authorize @user

        result = Users::Archive.call(@user)

        render json: { code: 422, errors: result.failure }, status: :unprocessable_entity unless result.success?
      end

      def reactivate
        authorize @user

        if @user.may_reactivate?
          @user.reactivate!
        else
          render json: { code: 422, errors: ['Пользователь не может быть восстановлен'] }, status: :unprocessable_entity
        end
      end

      def mass_update
        authorize :user, :mass_update?

        result = Users::MassUpdate.call(@users, params)

        if result.success?
          render :mass_update
        else
          render json: { code: 422, errors: result.failure }, status: :unprocessable_entity
        end
      end

      private

      def set_user
        @user = User.find(params[:id])
      end

      def set_users
        @users = User.where(id: params[:user_ids])
      end

      def user_params
        params.permit(:first_name, :last_name, :middle_name, :email, :phone, :role, :curator_id, :project_type, :gender)
      end
    end
  end
end
