# frozen_string_literal: true

module Api
  module Admin
    class WorkSchedulesController < BaseController
      before_action :set_calendar_month

      def show
        authorize :work_schedule, :show?
        @users = User.inspector.with_archived_by(@date).where(project_type: current_project_type).ordered
        @users = @users.where(curator: current_user) if current_user.curator?
        @calendar_days = @calendar_month.calendar_days.includes(visible_inspections: %i[checklist inspectable inspector]).group_by(&:user_id)
        render 'api/admin/schedules/show'
      end

      def create
        authorize :work_schedule, :create?
        if current_project_type != ProjectTypes::ALLOWED_TYPES[:policlinic]
          return render json: { code: 422, errors: { message: [I18n.t('errors.schedule.cannot_create_for_this_project')] } },
                        status: :unprocessable_entity
        end

        job_id = WorkSchedules::CreateJob.perform_later(@calendar_month, current_user).job_id
        render json: { code: 202, job_id: job_id }, status: :accepted
      end

      def update
        authorize :work_schedule, :update?
        job_id = WorkSchedules::UpdateJob.perform_later(@calendar_month, work_schedule_params, current_user).job_id
        render json: { code: 202, job_id: job_id }, status: :accepted
      end

      def destroy
        authorize :work_schedule, :destroy?
        job_id = WorkSchedules::CleanJob.perform_later(@calendar_month, current_user).job_id
        render json: { code: 202, job_id: job_id }, status: :accepted
      end

      private

      def set_calendar_month
        @date = begin
                  Date.strptime(params[:month], '%Y-%m')
                rescue StandardError
                  raise ActiveRecord::RecordNotFound
                end

        @calendar_month = CalendarMonth.find_or_initialize_by(date: @date, project_type: current_project_type)
        @calendar_month.save! if @calendar_month.new_record? && action_name != 'show'
      end

      def work_schedule_params
        params.permit(calendar_days: %i[user_id date kind working_hours])
      end
    end
  end
end
