# frozen_string_literal: true

module Api
  module Auth
    class PasswordsController < BaseController
      skip_before_action :authorize_by_access_cookie!
      skip_after_action :verify_authorized

      before_action :set_user, only: %i[check_token_expiration send_new_link reset]

      def forgot
        @user = User.where(state: %i[invited registered]).find_by!(email: params[:email]&.strip&.downcase)
        ::Auth::PasswordsMailer.forgot(@user).deliver_later
      end

      def check_token_expiration
        return unless @user.recovery_password_token_expired?

        render json: { code: 422, errors: { message: [I18n.t('errors.auth.recovery_password_token_expired')] } }, status: :unprocessable_entity
      end

      def reset
        result = ResetPassword.call(@user, params.to_unsafe_hash)

        if result.success?
          tokens = JWTSessions::Session.new(payload: { user_id: @user.id }, refresh_by_access_allowed: true).login
          set_access_cookie(tokens)
          @csrf = tokens[:csrf]
          render 'api/auth/sessions/create', status: :ok
        else
          render json: { code: 422, errors: result.failure }, status: :unprocessable_entity
        end
      end

      def send_new_link
        if @user.recovery_password_token_expired?
          action_name = @user.registered? ? :forgot : :invite
          ::Auth::PasswordsMailer.send(action_name, @user).deliver_later
        else
          render json: { code: 422, errors: { message: [I18n.t('errors.auth.recovery_password_token_not_expired_yet')] } },
                 status: :unprocessable_entity
        end
      end

      private

      def set_user
        @user = User.where.not(recovery_password_token: nil).find_by!(recovery_password_token: params[:token])
      end
    end
  end
end
