# frozen_string_literal: true

module Api
  class InspectionsController < ApplicationController
    before_action :set_inspection, only: %i[show update attach_file purge_file start finish]

    def index
      authorize :inspection, :index?

      @inspections = current_user.inspections.visible
                                 .includes(:inspectable, :checklist)
                                 .where(date: params[:date].presence || Time.zone.today)
                                 .order(:starts_at)
    end

    def show
      authorize @inspection
    end

    def update
      authorize @inspection

      result = Inspections::UpdateByInspector.call(@inspection, inspection_params, current_user)
      if result.success?
        render :show
      else
        render json: { code: 422, errors: result.failure }, status: :unprocessable_entity
      end
    end

    def attach_file
      authorize @inspection

      if @inspection.files.attach(file_params)
        render :show
      else
        render json: { code: 422, errors: @inspection.errors }, status: :unprocessable_entity
      end
    end

    def purge_file
      authorize @inspection
      attachment = @inspection.files.joins(:blob).find_by!('active_storage_blobs.key': params[:key])

      if attachment.purge
        head :no_content
      else
        render json: { errors: attachment.errors.full_messages }, status: :unprocessable_entity
      end
    end

    def start
      authorize @inspection
      @inspection.start!
    rescue AASM::InvalidTransition
      render json: { code: 422, errors: { message: [I18n.t("errors.inspection.cant_start.#{inspection.state}")] } }, status: :unprocessable_entity
    end

    def finish
      authorize @inspection
      @inspection.finish!
    rescue AASM::InvalidTransition
      render json: { code: 422, errors: { message: [I18n.t("errors.inspection.cant_finish.#{inspection.state}")] } }, status: :unprocessable_entity
    end

    private

    def file_params
      params.require(:signed_id)
    end

    def inspection_params
      params.permit(:raw_data, :state)
    end

    def set_inspection
      @inspection = Inspection.visible.find(params[:id])
    end
  end
end
