# frozen_string_literal: true

class ApplicationController < ActionController::API
  include JWTSessions::RailsAuthorization
  include Pagy::Backend
  include Pundit

  before_action :authorize_by_access_cookie!
  after_action  :verify_authorized

  rescue_from JWTSessions::Errors::Unauthorized, with: :not_authorized
  rescue_from Pagy::OverflowError, ActiveRecord::RecordNotFound, with: :not_found
  rescue_from Pundit::NotAuthorizedError, with: :forbidden
  rescue_from AASM::InvalidTransition, with: :please_contact_your_administrator

  helper_method :current_user

  private

  def current_user
    @current_user ||= begin
                        token = request_cookies[JWTSessions.cookie_by(:access)]
                        if JWTSessions::Session.new.session_exists?(token, :access)
                          payload = JWTSessions::Token.decode!(token).first
                          User.registered.find(payload['user_id'])
                        end
                      rescue ActiveRecord::RecordNotFound
                        raise JWTSessions::Errors::Unauthorized
                      end
  end

  def current_project_type
    @current_project_type ||= begin
      (current_user.superadmin? ? request_cookies[:project_type] : current_user.project_type).presence
    end || current_user.project_type
  end

  def not_authorized
    render json: { code: 401, errors: { message: [I18n.t('errors.auth.invalid_access_token')] } }, status: :unauthorized
  end

  def forbidden
    render json: { code: 403, errors: { message: [I18n.t('errors.forbidden')] } }, status: :forbidden
  end

  def not_found
    render json: { code: 404, errors: { message: [I18n.t('errors.not_found')] } }, status: :not_found
  end

  def please_contact_your_administrator(_e)
    # Airbrake.notify(e)
    render json: { code: 422, errors: { message: [I18n.t('errors.please_contact_your_administrator')] } }, status: :unprocessable_entity
  end

  def sortings
    @sortings ||= params[:sortings].to_a.reduce({}) { |acc, sorting| acc.merge(sorting[:field] => sorting[:value]) }
  end

  def filters
    @filters ||= params[:filters].to_a.reduce({}) { |acc, filter| acc.merge(filter[:field] => (filter[:value] || filter[:range])) }
  end

  # def pundit_params_for(_record)
  #   params
  # end
end
