# frozen_string_literal: true

class CalendarMonth < ApplicationRecord
  include AASM
  include ProjectTypes

  has_many :calendar_days, dependent: :restrict_with_exception
  has_many :inspections, through: :calendar_days

  validates :project_type, presence: true
  validates :date, presence: true, uniqueness: { scope: :project_type }

  aasm column: :state, requires_lock: true do # rubocop:disable Metrics/BlockLength
    state :initial, initial: true
    state :work_schedule_generating, :work_schedule_generated, :work_schedule_cleaning
    state :inspection_schedule_generating, :inspection_schedule_generated, :inspection_schedule_cleaning

    event :start_work_schedule_generation do
      transitions from: %i[initial], to: :work_schedule_generating
    end

    event :finish_work_schedule_generation do
      transitions from: %i[work_schedule_generating], to: :work_schedule_generated
    end

    event :start_work_schedule_cleaning do
      transitions from: %i[work_schedule_generated initial], to: :work_schedule_cleaning
    end

    event :finish_work_schedule_cleaning do
      transitions from: %i[work_schedule_cleaning], to: :initial
    end

    event :start_inspection_schedule_generation do
      transitions from: %i[work_schedule_generated initial], to: :inspection_schedule_generating
    end

    event :finish_inspection_schedule_generation do
      transitions from: %i[inspection_schedule_generating], to: :inspection_schedule_generated
    end

    event :start_inspection_schedule_cleaning do
      transitions from: %i[inspection_schedule_generated work_schedule_generated initial], to: :inspection_schedule_cleaning
    end

    event :finish_inspection_schedule_cleaning do
      transitions from: %i[inspection_schedule_cleaning], to: :work_schedule_generated
    end
  end

  def non_working_days
    NonWorkingDay.where(date: date.all_month)
  end

  def working_norm
    WorkingNorm.find_by(date: date)
  end
end
