# frozen_string_literal: true

class Inspection < ApplicationRecord
  include AASM

  CANCEL_REASONS = CalendarDay::EXTENDED_KINDS.merge(managment_resolution: 'managment_resolution').freeze

  searchkick

  enum cancel_reason: CANCEL_REASONS

  before_destroy :ensure_not_started

  belongs_to :inspector, class_name: 'User'
  belongs_to :inspectable, polymorphic: true
  belongs_to :checklist
  belongs_to :calendar_day

  has_one :calendar_month, through: :calendar_day

  has_many :inspection_results, dependent: :restrict_with_exception
  has_many_attached :files

  validate :starts_at_must_be_erlier_than_ends_at
  validates :date, presence: true

  scope :visible, -> { where(state: %i[planned started finished]) }
  scope :future, -> { where('date >= ?', Date.current) }
  scope :past, -> { where('date < ?', Date.current) }

  aasm column: :state do
    state :planned, initial: true
    state :started, :canceled, :finished, :remediation_planned

    event :start do
      transitions from: :planned, to: :started
    end

    event :cancel do
      transitions from: %i[planned started], to: :canceled
    end

    event :finish do
      transitions from: :started, to: :finished
    end

    event :plan_remediation do
      transitions from: :finished, to: :remediation_planned
    end
  end

  def search_data
    attributes.except('raw_data')
              .merge(inspectable_name: inspectable.name, checklist_name: checklist.name, project_type: inspectable_type.downcase)
              .merge(violated_parameter_ids: Inspections::ExtractViolatedParameterIds.call(self))
  end

  private

  def starts_at_must_be_erlier_than_ends_at
    return if starts_at.blank? || ends_at.blank?

    errors.add(:ends_at, I18n.t('activerecord.errors.messages.ends_at_cannot_be_earlier_than_starts_at')) if starts_at >= ends_at
  end

  def ensure_not_started
    errors.add(:state, I18n.t('activerecord.errors.messages.ensure_not_started')) unless planned? || canceled?
    throw(:abort) if errors.any?
  end
end
