# frozen_string_literal: true

class Policlinic < ApplicationRecord
  include Archivable

  geocoded_by :full_address

  searchkick

  belongs_to :district

  has_one :administrative_district, through: :district
  has_many :inspections, as: :inspectable, dependent: :restrict_with_exception
  has_many :repair_periods, as: :repairable, dependent: :restrict_with_exception

  scope :working_on, lambda { |date|
    non_working_policlinics = select(:id).joins(:repair_periods)
                                         .where('repair_periods.starts_at <= ? AND repair_periods.ends_at >= ?',
                                                date.beginning_of_day,
                                                date.end_of_day)
    where.not(id: non_working_policlinics)
  }

  SIZES_TO_MINUTES = {
    s: 90,
    m: 150,
    l: 210,
    xl: 270
  }.freeze

  SIZES = {
    s: 's',
    m: 'm',
    l: 'l',
    xl: 'xl'
  }.freeze

  KINDS = {
    adult: 'adult',
    children: 'children',
    adult_and_children: 'adult_and_children'
  }.freeze

  enum size: SIZES
  enum kind: KINDS

  validates :name, :address, :longitude, :latitude, :kind, :size, presence: true
  validates :name, uniqueness: true
  validates :time_to_inspect_minutes, numericality: { greater_than: 0 }

  def hours
    (time_to_inspect_minutes / 60.0).ceil(1)
  end

  def full_address
    "#{administrative_district.name}, #{district.name}, #{address}"
  end

  def search_data
    attributes.merge(
      administrative_district_id: administrative_district.id,
      district: { name: district.name },
      administrative_district: { name: administrative_district.name }
    )
  end
end
