# frozen_string_literal: true

class User < ApplicationRecord # rubocop:disable Metrics/ClassLength
  include AASM
  include ProjectTypes

  ROLES = {
    inspector: 'inspector',
    curator: 'curator',
    admin: 'admin',
    superadmin: 'superadmin'
  }.freeze

  GENDERS = {
    male: 'male',
    female: 'female'
  }.freeze

  MIN_PASSWORD_LENGTH = 8
  MIN_RECOVERY_TOKEN_LENGTH = 32
  RECOVERY_PASSWORD_TOKEN_EXP_TIME = 2.days

  enum role: ROLES
  enum gender: GENDERS

  searchkick

  belongs_to :curator, optional: true, class_name: 'User'

  has_secure_password
  has_many :inspectors, -> { not_archived },
           inverse_of: :curator,
           class_name: 'User',
           foreign_key: 'curator_id',
           dependent: :restrict_with_exception
  has_many :calendar_days, dependent: :restrict_with_exception
  has_many :inspections, dependent: :restrict_with_exception, foreign_key: :inspector_id, inverse_of: :inspector
  has_many :vacations, dependent: :restrict_with_exception

  validates :password, length: { minimum: MIN_PASSWORD_LENGTH }, if: :password_digest_changed?
  validates :email, presence: true, uniqueness: true, email: true, unless: :archived?
  validates :phone, uniqueness: true, length: { maximum: 11 }, allow_nil: true
  validates :first_name, :last_name, :role, :project_type, presence: true
  validates :first_name, :middle_name, :last_name, length: { maximum: 50 }
  validates :recovery_password_token, length: { minimum: MIN_RECOVERY_TOKEN_LENGTH }, uniqueness: true, allow_nil: true

  before_validation -> { self.email = email&.strip&.downcase }
  before_validation :format_phone

  scope :not_archived, -> { where.not(state: :archived) }
  scope :ordered, lambda {
                    left_joins(:curator)
                      .order('curators_users.last_name, curators_users.first_name, curators_users.middle_name,
                             users.last_name, users.first_name, users.middle_name')
                  }
  scope :with_archived_by, lambda { |date|
    where(
      '(users.created_at < :start_month and users.archived_at > :end_month)' \
      ' or (users.archived_at BETWEEN :start_month AND :end_month)' \
      ' or (users.created_at < :end_month and users.archived_at is null)',
      start_month: date.beginning_of_month, end_month: date.end_of_month
    )
  }

  aasm column: :state do
    state :invited, initial: true
    state :registered, :archived

    event :switch_to_registered do
      transitions from: %i[invited registered], to: :registered
    end

    event :archive do
      transitions from: %i[invited registered], to: :archived
    end

    event :reactivate do
      transitions from: :archived, to: :registered
    end
  end

  def first_and_middle_name
    first_name.to_s + (middle_name.present? ? " #{middle_name}" : '')
  end

  def short_full_name
    "#{last_name} #{first_name[0]}." + (middle_name.present? ? " #{middle_name[0]}." : '').to_s
  end

  def full_name
    "#{last_name} #{first_name}" + (middle_name.present? ? " #{middle_name}" : '')
  end

  def recovery_password_token_expired?
    RECOVERY_PASSWORD_TOKEN_EXP_TIME.since(recovery_password_token_sent_at) < Time.current
  end

  def generate_recovery_password_link
    retries ||= 0
    recovery_password_token = SecureRandom.hex(MIN_RECOVERY_TOKEN_LENGTH)
    update!(recovery_password_token_sent_at: Time.current, recovery_password_token: recovery_password_token)
    "#{ENV.fetch('FRONTEND_URL')}/auth/recovery-password?token=#{recovery_password_token}"
  rescue ActiveRecord::RecordNotUnique, ActiveRecord::RecordInvalid
    retry if (retries += 1) < 5
    raise # TODO: notify through Airbrake
  end

  def set_password
    self.password = SecureRandom.alphanumeric(8)
  end

  def search_data
    attributes.merge(full_name: full_name, curator: { full_name: curator&.full_name })
  end

  def admin_of_project?(project)
    admin? && project_type == project
  end

  def curator_of?(inspector)
    curator? && inspector&.curator_id == id
  end

  private

  def format_phone
    ru_phone = RussianPhone::Number.new(phone, default_country: 7)
    self.phone = ru_phone.clean.presence
  end
end
