# frozen_string_literal: true

module CalendarDays
  class Update < Base
    def initialize(calendar_day, params, current_user)
      @calendar_day = calendar_day
      @params = params.is_a?(ActionController::Parameters) ? params.permit!.to_h : HashWithIndifferentAccess.new(params)
      @current_user = current_user
    end

    def call
      yield find_target_user
      yield authorize_curator if current_user.curator?
      yield update_day

      after_commit do
        cancel_inspections if need_cancel_inspections?
        destroy_inspections if need_destroy_inspections?
        broadcast_changes
      end

      Success(calendar_day)
    end

    private

    attr_reader :calendar_day, :params, :current_user
    attr_accessor :target_user

    def find_target_user
      self.target_user = calendar_day.user
      Success()
    end

    def authorize_curator
      if current_user.curator_of?(target_user)
        Success()
      else
        Failure(message: [I18n.t('errors.cant_update.calendar_day.because_user_is_not_curator_of_target_user')])
      end
    end

    def update_day
      if calendar_day.update(calendar_day_params)
        Success()
      else
        Failure(message: calendar_day.errors.full_messages)
      end
    end

    def need_cancel_inspections?
      calendar_day.inspections.planned.present? && params[:cancel_inspections] && params[:cancel_reason].present?
    end

    def cancel_inspections
      Inspections::Cancel.call(calendar_day.inspections.planned, cancel_reason: params[:cancel_reason])
    end

    def need_destroy_inspections?
      calendar_day.inspections.planned.present? && params[:destroy_inspections]
    end

    def destroy_inspections
      calendar_day.inspections.planned.destroy_all
    end

    def calendar_day_params
      @calendar_day_params ||= params.select { |key, _| permit_attributes.include?(key) }
    end

    def permit_attributes
      %w[extended_kind]
    end

    def broadcast_changes
      broadcast_to_admins
      broadcast_to_curator
    end

    def broadcast_to_admins
      ActionCable.server.broadcast(
        "admin:schedule:#{calendar_day.calendar_month.project_type}",
        action: 'calendar_days_update',
        date: calendar_day.calendar_month.date,
        calendarDays: CamelizedPartialRenderer.call(
          partial: 'api/admin/calendar_days/ws_calendar_days_with_inspections',
          locals: { calendar_days: [calendar_day] }
        )
      )
    end

    def broadcast_to_curator
      return if calendar_day.user.curator.blank?

      ActionCable.server.broadcast(
        "admin:schedule:#{calendar_day.calendar_month.project_type}:#{calendar_day.user.curator.id}",
        action: 'calendar_days_update',
        date: calendar_day.calendar_month.date,
        calendarDays: CamelizedPartialRenderer.call(
          partial: 'api/admin/calendar_days/ws_calendar_days_with_inspections',
          locals: { calendar_days: [calendar_day] }
        )
      )
    end
  end
end
