# frozen_string_literal: true

module Checklists
  class Create
    include Dry::Monads[:result, :do]

    static_facade :call
    pattr_initialize :params, :current_user

    def call
      yield authorize_project_type
      ActiveRecord::Base.transaction do
        yield create_entities_from_raw_data
        yield create_checklist
      end

      Success(checklist)
    end

    private

    attr_accessor :checklist, :parsed_checklist_raw_data

    def authorize_project_type
      return Success() if current_user.superadmin?
      return Success() if current_user.admin_of_project?(params[:project_type])

      Failure(message: [I18n.t('errors.cant_create.checklist.for_another_project')])
    end

    def create_entities_from_raw_data
      self.parsed_checklist_raw_data = JSON[params[:raw_data]]

      parsed_checklist_raw_data['blocks']&.each do |raw_block|
        created_block = Block.find_or_create_by!(id: raw_block['uid']) do |block|
          block.id = nil
          block.title = raw_block['title']
        end
        raw_block['uid'] = created_block.id

        raw_block['groups']&.each do |raw_group|
          if raw_group['title'].present?
            created_group = Group.find_or_create_by!(id: raw_group['uid']) do |group|
              group.id = nil
              group.title = raw_group['title']
            end
            raw_group['uid'] = created_group.id
          end

          raw_group['parameters']&.each do |raw_parameter|
            created_parameter = Parameter.find_or_create_by!(id: raw_parameter['uid']) do |parameter|
              parameter.id = nil
              parameter.title = raw_parameter['title']
            end
            raw_parameter['uid'] = created_parameter.id
          end
        end
      end

      params[:raw_data] = parsed_checklist_raw_data.to_json
      Success()
    rescue ActiveRecord::RecordInvalid => e
      Failure(message: e.record.errors.full_messages)
    end

    def create_checklist
      self.checklist = Checklist.new(params)
      return Success(checklist) if checklist.save

      Failure(message: checklist.errors.full_messages)
    end
  end
end
