# frozen_string_literal: true

module Checklists
  class Update
    include Dry::Monads[:result, :do]

    static_facade :call
    pattr_initialize :checklist, :params, :current_user

    def call
      yield authorize_project_type if project_type_changed?
      ActiveRecord::Base.transaction do
        yield update_entities_from_raw_data
        yield update_checklist
      end

      Success(checklist)
    end

    private

    attr_accessor :parsed_checklist_raw_data

    def authorize_project_type
      return Success() if current_user.superadmin?
      return Success() if current_user.admin_of_project?(params[:project_type])

      Failure(message: [I18n.t('errors.cant_update.checklist.for_another_project')])
    end

    def project_type_changed?
      params[:project_type].present? && (params[:project_type] != checklist.project_type)
    end

    def update_entities_from_raw_data
      self.parsed_checklist_raw_data = JSON[params[:raw_data]]

      parsed_checklist_raw_data['blocks']&.each do |raw_block|
        created_block = Block.find_or_create_by!(id: raw_block['uid']) do |block|
          block.title = raw_block['title']
        end
        created_block.update(title: raw_block['title'])
        raw_block['uid'] = created_block.id

        raw_block['groups']&.each do |raw_group|
          if raw_group['title'].present?
            created_group = Group.find_or_create_by!(id: raw_group['uid']) do |group|
              group.title = raw_group['title']
            end
            created_group.update(title: raw_group['title'])
            raw_group['uid'] = created_group.id
          end

          raw_group['parameters']&.each do |raw_parameter|
            created_parameter = Parameter.find_or_create_by!(id: raw_parameter['uid']) do |parameter|
              parameter.title = raw_parameter['title']
            end
            created_parameter.update(title: raw_parameter['title'])
            raw_parameter['uid'] = created_parameter.id
          end
        end
      end

      params[:raw_data] = parsed_checklist_raw_data.to_json
      Success()
    rescue ActiveRecord::RecordInvalid => e
      Failure(message: e.record.errors.full_messages)
    end

    def update_checklist
      return Success(checklist) if checklist.update(params)

      Failure(message: checklist.errors.full_messages)
    end
  end
end
