# frozen_string_literal: true

module Schedulable
  def lock_and_rollback_state_on_fail(calendar_month, ttl:, error_messages: [])
    lock(calendar_month, ttl: ttl, error_messages: error_messages) do
      prev_state = calendar_month.state
      yield
    rescue StandardError => e
      rollback_to_prev_state(calendar_month, prev_state)
      raise e
    end
  end

  def lock(calendar_month, ttl:, error_messages: [])
    ttl_in_ms = ttl.to_i * 1000
    LockManager.instance.lock("schedule:#{calendar_month.id}", ttl_in_ms) do |locked|
      return Failure(message: error_messages) unless locked

      yield
    end
  end

  def rollback_to_prev_state(calendar_month, prev_state)
    calendar_month.update!(state: prev_state)
  end

  def broadcast_to_subscribers(action:, calendar_month:) # rubocop:disable Metrics/MethodLength
    ActionCable.server.broadcast(
      "admin:schedule:#{calendar_month.project_type}",
      action: action,
      date: calendar_month.date,
      state: calendar_month.state
    )
    User.curator.not_archived.pluck(:id).each do |curator_id|
      ActionCable.server.broadcast(
        "admin:schedule:#{calendar_month.project_type}:#{curator_id}",
        action: action,
        date: calendar_month.date,
        state: calendar_month.state
      )
    end
  end

  # when interacting with schedulable we can only operate on future days
  def available_days
    if calendar_month.date.all_month.cover?(Date.current)
      Date.current..calendar_month.date.end_of_month
    else
      calendar_month.date.all_month
    end
  end

  def ensure_not_cleaning_passed_month
    if calendar_month.date.end_of_month < Date.current
      Failure(message: [I18n.t('errors.schedule.cannot_clean_for_passed_month')])
    else
      Success()
    end
  end
end
