# frozen_string_literal: true

module FileArchives
  class CreateZip
    include Dry::Monads[:result, :do]

    static_facade :call
    pattr_initialize :inspection

    def call # rubocop:disable Metrics/AbcSize, Metrics/MethodLength
      yield ensure_files_exist
      yield set_or_create_archive_folder
      yield set_zipfile_path

      return Success(zipfile_path) if File.exist?(zipfile_path)

      LockManager.instance.lock("file_archive:#{inspection.id}", 10.minutes.in_milliseconds) do |locked|
        return Failure('Уже в процессе архивации') unless locked

        yield create_zipfile
        yield download_files
        yield cleanup
      end

      Success(zipfile_path)
    end

    private

    attr_accessor :archive_folder, :zipfile_path, :zipfile, :tempfiles

    def ensure_files_exist
      return Failure('Нет файлов для скачивания') unless inspection.files.exists?

      Success()
    end

    def set_or_create_archive_folder
      self.archive_folder = Rails.root.join("tmp/file_archives/#{inspection.id}")

      FileUtils.mkdir_p(archive_folder) unless Dir.exist?(archive_folder)

      Success()
    end

    def set_zipfile_path
      zipfile_name = "Медиафайлы_по_проверке_id_#{inspection.id}.zip"
      self.zipfile_path = File.join(archive_folder, zipfile_name)

      Success()
    end

    def create_zipfile
      self.zipfile = Zip::File.open(zipfile_path, Zip::File::CREATE)

      return Failure('Не удалось создать zip файл') unless zipfile

      Success()
    end

    def download_files # rubocop:disable Metrics/MethodLength
      self.tempfiles = []
      inspection.files.each do |file|
        filename = file.blob.filename.to_s
        filepath = File.join(archive_folder, filename)

        File.open(filepath, 'wb') do |f|
          file.download { |chunk| f.write(chunk) }
        end

        duplicate = zipfile.find_entry(filename)
        filename_to_write = duplicate ? (Random.uuid + '-' + filename) : filename
        zipfile.add(filename_to_write, filepath)
        tempfiles << filepath
      end

      Success()
    end

    def cleanup
      zipfile.close
      tempfiles.each do |filepath|
        FileUtils.rm_rf(filepath)
      end

      Success()
    end
  end
end
