# frozen_string_literal: true

module InspectionSchedules
  class Create
    include Dry::Monads[:result, :do]
    include AfterCommitEverywhere
    include Schedulable

    static_facade :call
    pattr_initialize :calendar_month, :checklist

    def call
      lock_and_rollback_state_on_fail(calendar_month, ttl: 5.minutes, error_messages: [I18n.t('errors.schedule.cannot_create')]) do
        yield set_inspectable_type
        yield ensure_no_inspections
        yield start_inspection_schedule_generation(calendar_month)
        broadcast_to_subscribers(action: :state_update, calendar_month: calendar_month)

        ActiveRecord::Base.transaction do
          yield destroy_canceled_inspections
          yield generate_inspections
          yield finish_inspection_schedule_generation(calendar_month)

          after_commit do
            broadcast_to_subscribers(action: :reload, calendar_month: calendar_month)
          end
        end
      end
      Success()
    end

    private

    attr_accessor :inspectable_type

    def set_inspectable_type
      self.inspectable_type = calendar_month.project_type.camelize.safe_constantize
      Success()
    end

    def ensure_no_inspections
      inspections = calendar_month.inspections.where.not(state: %i[canceled])
      if inspections.exists?
        Failure(message: [I18n.t('errors.inspection_schedule.cannot_create_because_inspections_present')])
      else
        Success()
      end
    end

    def start_inspection_schedule_generation(calendar_month)
      calendar_month.start_inspection_schedule_generation!
      Success()
    rescue AASM::InvalidTransition
      Failure(message: [I18n.t('errors.schedule.cannot_create')])
    end

    def destroy_canceled_inspections
      Inspection.where(calendar_day: calendar_month.calendar_days, state: %i[canceled]).destroy_all
      Success()
    end

    def generate_inspections
      calendar_month.calendar_days.where(date: available_days).order(:date, :user_id).each do |calendar_day|
        working_hours = calendar_day.working_hours.hours
        total_hours = 0.hours

        last = nil
        inspectable_count = 0

        while total_hours < working_hours
          time_left = working_hours - total_hours
          inspectable = find_inspectable(last, time_left, calendar_day)

          break if inspectable.nil?

          inspectable_count += 1

          distance = last.present? ? last.distance_to(inspectable, :km) : 0
          distance_in_minutes = (distance * 10).minutes
          distance_in_minutes = distance_in_minutes < 15.minutes ? 15.minutes : distance_in_minutes
          total_hours += distance_in_minutes

          break if total_hours > working_hours

          starts_at = calendar_day.date.to_datetime + 5.hours + total_hours
          starts_at += 1.hour if inspectable_count > 2

          Inspection.create(
            inspectable: inspectable,
            starts_at: starts_at,
            date: calendar_day.date,
            inspector: calendar_day.user,
            checklist: checklist,
            calendar_day: calendar_day
          )

          last = inspectable
          total_hours += inspectable.hours.hours
        end
      end
      Success()
    end

    def finish_inspection_schedule_generation(calendar_month)
      calendar_month.finish_inspection_schedule_generation!
      Success()
    end

    def find_inspectable(from, time_left, calendar_day)
      scope = apply_base_scope(calendar_day)

      if from.blank?
        scope = group_by_inspections_count(scope, :asc)
      else
        scope = if time_left >= 4.5.hours
                  from.nearbys(20)
                elsif time_left >= 3.5.hours
                  from.nearbys(20).where(size: %w[l m s])
                elsif time_left >= 2.5.hours
                  from.nearbys(20).where(size: %w[m s])
                elsif time_left >= 1.5.hours
                  from.nearbys(20).where(size: ['s'])
                else
                  inspectable_type.none
                end

        scope = scope.where.not(id: group_by_inspections_count(scope, :desc).limit(limit_inspectable).map(&:id))
      end

      scope = skip_twice_visited_by_inspector(scope, calendar_day)
      scope = skip_today_visited(scope, calendar_day).first
    end

    def apply_base_scope(calendar_day)
      inspectable_type.where(id: inspectable_type.working_on(calendar_day.date)).not_archived
    end

    def group_by_inspections_count(scope, order)
      scope.left_joins(:inspections).select("#{inspectable_type.model_name.plural}.*, count(inspections.id) as inspection_count").group("#{inspectable_type.model_name.plural}.id").order(inspection_count: order)
    end

    def skip_twice_visited_by_inspector(scope, calendar_day)
      scope.where.not(id: calendar_day.user.inspections.where(inspectable_type: inspectable_type.model_name.human).where('date > ? and date < ?', 30.days.ago, Time.current).pluck(:inspectable_id))
    end

    def skip_today_visited(scope, calendar_day)
      scope.where.not(id: Inspection.where(inspectable_type: inspectable_type.model_name.human).where(date: calendar_day.date.beginning_of_day..calendar_day.date.end_of_day).pluck(:inspectable_id))
    end

    def limit_inspectable
      @average ||= inspectable_type.not_archived.count / 5
    end
  end
end
