# frozen_string_literal: true

module Inspections
  class Base
    include Dry::Monads[:result, :do]
    include AfterCommitEverywhere

    static_facade :call

    private

    def authorize_target_inspector
      target_inspector = User.find_by(id: params[:inspector_id])
      return Success(target_inspector) unless current_user.curator?

      if (target_inspector.present? && current_user.curator_of?(target_inspector)) || (current_user.curator? && current_user == target_inspector)
        Success(target_inspector)
      else
        Failure(message: [I18n.t('errors.inspection.cannot_set_inspector')])
      end
    end

    def verify_project_type(operation_name)
      if !current_user.superadmin? && current_user.project_type != params[:inspectable_type]
        Failure(message: [I18n.t("errors.cant_#{operation_name}.inspection.for_another_project")])
      else
        Success()
      end
    end

    def verify_checklist
      if checklist.project_type == params[:inspectable_type]
        Success()
      else
        Failure(checklist_id: [I18n.t('errors.cant_create.inspection.project_type_mismatch')])
      end
    end

    def ensure_inspectable_not_repair
      if inspectable.repair_periods.where("(?, INTERVAL '1 days') OVERLAPS (starts_at, ends_at)", params[:date]).exists?
        Failure(message: [I18n.t('errors.cant_create.inspection.because_overlapping_repair_period')])
      else
        Success()
      end
    end

    def find_inspectable
      if ProjectTypes::ALLOWED_TYPES.value?(params[:inspectable_type])
        inspectable_type = params[:inspectable_type].capitalize.constantize
        self.inspectable = inspectable_type.find(params[:inspectable_id])
        Success()
      else
        Failure(inspectable: [I18n.t('errors.cant_create.inspection.project_type_not_found')])
      end
    rescue ActiveRecord::RecordNotFound
      Failure(inspectable: [I18n.t('errors.cant_create.inspection.project_type_not_found')])
    end

    def find_checklist
      self.checklist = Checklist.find(params[:checklist_id])
      Success()
    rescue ActiveRecord::RecordNotFound
      Failure(checklist_id: [I18n.t('errors.cant_create.inspection.checklist_not_found')])
    end

    def permitted_params
      params.permit(:inspector_id, :checklist_id, :date, :starts_at, :ends_at, :raw_data, :state)
    end

    def find_calendar_day(date)
      calendar_day = CalendarDay.working.find_by(user_id: permitted_params[:inspector_id], date: date)
      Success(calendar_day)
    end

    def find_or_create_calendar_day(date)
      calendar_month = CalendarMonth.find_or_create_by(date: date.beginning_of_month, project_type: :policlinic)
      calendar_day = CalendarDay.working.find_or_create_by!(calendar_month: calendar_month, user_id: permitted_params[:inspector_id], date: date)
      Success(calendar_day)
    end

    def persist(calendar_day)
      inspection.calendar_day = calendar_day
      if inspection.save
        Success(inspection)
      else
        Failure(message: inspection.errors.full_messages)
      end
    end

    def broadcast_to_admins(action:, inspection:)
      ActionCable.server.broadcast(
        "admin:schedule:#{inspection.calendar_month.project_type}",
        action: action,
        date: inspection.calendar_month.date,
        inspections: CamelizedPartialRenderer.call(
          partial: 'api/admin/inspections/ws_inspections',
          locals: { inspections: [inspection] }
        )
      )
    end

    def broadcast_to_curator(action:, inspection:, curator_id:)
      return if curator_id.blank?

      ActionCable.server.broadcast(
        "admin:schedule:#{inspection.calendar_month.project_type}:#{inspection.inspector.curator_id}",
        action: action,
        date: inspection.calendar_month.date,
        inspections: CamelizedPartialRenderer.call(
          partial: 'api/admin/inspections/ws_inspections',
          locals: { inspections: [inspection] }
        )
      )
    end
  end
end
