# frozen_string_literal: true

module Inspections
  class Cancel < Base
    def initialize(inspections, cancel_reason: nil)
      @inspections = inspections.respond_to?(:each) ? inspections : [inspections]
      @cancel_reason = cancel_reason || Inspection::CANCEL_REASONS[:managment_resolution]
    end

    def call
      ActiveRecord::Base.transaction do
        yield ensure_allowed_cancel_reason
        yield cancel_inspections

        after_commit do
          broadcast_changes
        end
      end

      Success(inspections)
    end

    private

    attr_reader :inspections, :cancel_reason

    def ensure_allowed_cancel_reason
      return Success() if Inspection::CANCEL_REASONS.value?(cancel_reason)

      Failure(message: [I18n.t('errors.inspection.not_allowed_cancel_reason')])
    end

    def cancel_inspections
      inspections.each do |inspection|
        cancel_inspection(inspection)
      end

      Success()
    end

    def cancel_inspection(inspection)
      return unless inspection.planned? || inspection.started?

      inspection.cancel_reason = cancel_reason
      inspection.cancel!
    end

    def broadcast_changes
      inspections.each do |inspection|
        broadcast_to_admins(action: 'inspections_update', inspection: inspection)
        broadcast_to_curator(action: 'inspections_update', inspection: inspection, curator_id: inspection.inspector.curator_id)
      end
    end
  end
end
