# frozen_string_literal: true

class ResetPassword
  include Dry::Monads[:result, :do]

  Validator = Dry::Schema.Params do
    required(:password).filled
  end

  def self.call(user, params)
    new(user, params).call
  end

  def initialize(user, params)
    @user = user
    @params = params
  end

  def call
    ActiveRecord::Base.transaction do
      yield validate
      yield check_token_expiration
      yield update_password
      yield switch_to_registered
    end
    Success(user)
  end

  private

  attr_reader :user, :params

  def validate
    result = Validator.call(params)
    result.success? ? Success() : Failure(errors: result.errors.to_h)
  end

  def check_token_expiration
    user.recovery_password_token_expired? ? Failure(message: [I18n.t('errors.auth.recovery_password_token_expired')]) : Success()
  end

  def update_password
    if user.update(password: params[:password].strip, recovery_password_token: nil, recovery_password_token_sent_at: nil)
      Success(user)
    else
      Failure(user.errors)
    end
  end

  def switch_to_registered
    user.switch_to_registered!
    Success(user)
  rescue AASM::InvalidTransition
    Failure(message: [I18n.t('errors.please_contact_your_administrator')])
  end
end
