# frozen_string_literal: true

module Users
  class MassUpdate < Base
    static_facade :call

    def initialize(users, params)
      @users = users
      @params = params.is_a?(ActionController::Parameters) ? params.permit!.to_h : HashWithIndifferentAccess.new(params)
    end

    def call
      yield ensure_purge_starts_at_not_passed if params[:purge_starts_at].present?
      ActiveRecord::Base.transaction do
        yield update_users
      end

      Success()
    end

    private

    attr_reader :users, :params

    def ensure_purge_starts_at_not_passed
      purge_date = params[:purge_starts_at].is_a?(String) ? Date.parse(params[:purge_starts_at]) : params[:purge_starts_at]
      if purge_date >= Date.current
        Success()
      else
        Failure(message: [I18n.t('errors.cant_update.user.purge_date_passed')])
      end
    end

    def update_users
      users.each do |user|
        if user.update(user_params)
          cancel_planned_inspection(user) if cancel_inspections?
          destroy_planned_inspection(user) if destroy_inspections?
        end
      end
      Success()
    end

    def cancel_planned_inspection(user)
      Inspections::Cancel.call(
        user.inspections.planned.where('date >= ?', params[:purge_starts_at]),
        cancel_reason: Inspection::CANCEL_REASONS[:managment_resulutions]
      )
    end

    def destroy_planned_inspection(user)
      user.inspections.planned.where('date >= ?', params[:purge_starts_at]).each(&:destroy)
    end

    def cancel_inspections?
      params[:cancel_inspections] && params[:purge_starts_at].present?
    end

    def destroy_inspections?
      params[:destroy_inspections] && params[:purge_starts_at].present?
    end

    def user_params
      @user_params ||= params.select { |key, _| permit_attributes.include?(key) }
    end

    def permit_attributes
      %w[project_type curator_id]
    end
  end
end
