# frozen_string_literal: true

module Vacations
  class Create
    include Dry::Monads[:result, :do]

    static_facade :call
    pattr_initialize :current_user, :target_user, :params
    attr_accessor :vacation

    def call # rubocop:disable Metrics/MethodLength
      yield ensure_current_user_curator_of_target_user if current_user.curator?
      yield ensure_current_user_admin_of_target_user_project if current_user.admin?
      yield ensure_no_planned_inspections
      yield remove_non_working_days
      yield ensure_no_overlapping_vacations
      yield ensure_user_has_no_other_calendar_days

      ActiveRecord::Base.transaction do
        yield create_vacation
        yield create_calendar_days
      end

      Success(vacation)
    end

    private

    def ensure_current_user_curator_of_target_user
      if current_user.curator_of?(target_user)
        Success(vacation)
      else
        Failure(message: [I18n.t('errors.cant_create.vacation.because_user_is_not_curator_of_updating_user')])
      end
    end

    def ensure_current_user_admin_of_target_user_project
      if current_user.admin_of_project?(target_user.project_type)
        Success(vacation)
      else
        Failure(message: [I18n.t('errors.cant_create.vacation.for_another_project')])
      end
    end

    def ensure_no_planned_inspections
      if target_user.inspections.visible.exists?(starts_at: vacation_period)
        Failure(message: [I18n.t('errors.cant_create.vacation.because_planned_inspections')])
      else
        Success(vacation)
      end
    end

    def remove_non_working_days
      target_user.calendar_days.where(date: vacation_period, kind: CalendarDay::KINDS[:user_non_working]).destroy_all
      Success()
    end

    def ensure_no_overlapping_vacations
      if target_user.vacations.where('(?, ?) OVERLAPS (starts_at, ends_at)', params[:starts_at], params[:ends_at]).exists?
        Failure(message: [I18n.t('errors.cant_create.vacation.because_overlapping_vacation')])
      else
        Success(vacation)
      end
    end

    def ensure_user_has_no_other_calendar_days
      if target_user.calendar_days.exists?(date: vacation_period)
        Failure(message: [I18n.t('errors.cant_create.vacation.because_user_has_other_days')])
      else
        Success(vacation)
      end
    end

    def create_vacation
      self.vacation = Vacation.new(params)
      if vacation.save
        Success(vacation)
      else
        Failure(message: vacation.errors.full_messages)
      end
    end

    def create_calendar_days
      calendar_days = vacation.calendar_days.create((vacation.starts_at..vacation.ends_at).map do |e|
        calendar_month = CalendarMonth.find_or_create_by!(date: e.beginning_of_month, project_type: vacation.user.project_type)
        { date: e, user: vacation.user, calendar_month: calendar_month }
      end)
      if calendar_days.all?(&:persisted?)
        Success(vacation)
      else
        Failure(message: calendar_days.map(&:errors).map(&:full_messages).flatten)
      end
    end

    def vacation_period
      @vacation_period ||= params[:starts_at]..params[:ends_at]
    end
  end
end
