# frozen_string_literal: true

module Vacations
  class Update
    include Dry::Monads[:result, :do]

    static_facade :call
    pattr_initialize :user, :vacation, :params

    def call
      yield ensure_no_planned_inspections
      yield ensure_no_overlapping_vacations

      ActiveRecord::Base.transaction do
        yield update_vacation
        yield reassign_calendar_days unless period_changed? || !user_changed?
        yield destroy_old_calendar_days if period_changed?
        yield create_new_calendar_days if period_changed?
      end

      Success(vacation)
    end

    private

    def ensure_no_planned_inspections
      if user.inspections.exists?(starts_at: vacation_period)
        Failure(message: [I18n.t('errors.cant_update.vacation.because_planned_inspections')])
      else
        Success(vacation)
      end
    end

    def ensure_no_overlapping_vacations
      if user.vacations
             .where.not(id: vacation.id)
             .where('starts_at >= ? AND starts_at <= ?', params[:starts_at], params[:ends_at]).any?
        Failure(message: [I18n.t('errors.cant_update.vacation.because_overlapping_vacation')])
      else
        Success(vacation)
      end
    end

    def update_vacation
      if vacation.update(params)
        Success(vacation)
      else
        Failure(message: vacation.errors.full_messages)
      end
    end

    def reassign_calendar_days
      if vacation.calendar_days.update(user: vacation.user)
        Success(vacation)
      else
        Failure(message: [I18n.t('errors.cant_update.vacation.couldnt_reassign_calendar_days')])
      end
    end

    def user_changed?
      vacation.saved_change_to_user_id?
    end

    def period_changed?
      vacation.saved_change_to_starts_at? ||
        vacation.saved_change_to_ends_at?
    end

    def destroy_old_calendar_days
      if vacation.calendar_days.destroy_all
        Success(vacation)
      else
        Failure(message: [I18n.t('errors.cant_update.vacation.couldnt_destroy_calendar_days')])
      end
    end

    def create_new_calendar_days
      calendar_days = vacation.calendar_days.create((vacation.starts_at..vacation.ends_at).map do |e|
        calendar_month = CalendarMonth.find_or_create_by!(date: e.beginning_of_month, project_type: vacation.user.project_type)
        { date: e, user: vacation.user, calendar_month: calendar_month }
      end)
      if calendar_days.all?(&:persisted?)
        Success(vacation)
      else
        Failure(message: calendar_days.map(&:errors).map(&:full_messages).flatten)
      end
    end

    def vacation_period
      @vacation_period ||= params[:starts_at]..params[:ends_at]
    end
  end
end
