# frozen_string_literal: true

module WorkSchedules
  class Clean
    include Dry::Monads[:result, :do]
    include AfterCommitEverywhere
    include Schedulable

    static_facade :call
    pattr_initialize :calendar_month

    def call # rubocop:disable Metrics/MethodLength
      lock_and_rollback_state_on_fail(calendar_month, ttl: 2.minutes, error_messages: [I18n.t('errors.schedule.cannot_clean')]) do
        yield ensure_not_cleaning_passed_month
        yield start_work_schedule_cleaning(calendar_month)
        broadcast_to_subscribers(action: :state_update, calendar_month: calendar_month)

        ActiveRecord::Base.transaction do
          yield destroy_calendar_days
          yield finish_work_schedule_cleaning(calendar_month)

          after_commit do
            broadcast_to_subscribers(action: :reload, calendar_month: calendar_month)
          end
        end
      end
      Success()
    end

    private

    def start_work_schedule_cleaning(calendar_month)
      calendar_month.start_work_schedule_cleaning!
      Success()
    rescue AASM::InvalidTransition
      Failure(message: [I18n.t('errors.schedule.cannot_clean')])
    end

    def destroy_calendar_days
      calendar_month.calendar_days
                    .where(date: available_days)
                    .where.not(kind: CalendarDay::KINDS.values_at(:vacation))
                    .destroy_all

      Success()
    end

    def finish_work_schedule_cleaning(calendar_month)
      calendar_month.finish_work_schedule_cleaning!
      Success()
    end
  end
end
