# frozen_string_literal: true

module WorkSchedules
  class Create # rubocop:disable Metrics/ClassLength
    include Dry::Monads[:result, :do]
    include AfterCommitEverywhere
    include Schedulable

    PATTERNS = [
      [1, 2, 4, 5],
      [2, 3, 5, 6]
    ].freeze

    WORKING_HOURS = 11.25

    static_facade :call
    pattr_initialize :calendar_month

    def call # rubocop:disable Metrics/MethodLength, Metrics/AbcSize
      lock_and_rollback_state_on_fail(calendar_month, ttl: 2.minutes, error_messages: [I18n.t('errors.schedule.cannot_create')]) do
        yield ensure_not_generating_for_passed_month
        yield ensure_presence_of_working_norm
        yield start_work_schedule_generation(calendar_month)
        broadcast_to_subscribers(action: :state_update, calendar_month: calendar_month)

        ActiveRecord::Base.transaction do
          yield destroy_user_non_working_calendar_days
          yield generate_calendar_days(calendar_month)
          yield finish_work_schedule_generation(calendar_month)

          after_commit do
            broadcast_to_subscribers(action: :reload, calendar_month: calendar_month)
          end
        end
      end
      Success()
    end

    private

    def ensure_not_generating_for_passed_month
      if calendar_month.date.end_of_month < Date.current
        Failure(message: [I18n.t('errors.schedule.cannot_create_for_passed_month')])
      else
        Success()
      end
    end

    def ensure_presence_of_working_norm
      if calendar_month.working_norm.present?
        Success()
      else
        Failure(message: [I18n.t('errors.schedule.cannot_create_because_working_hours_norm_not_set')])
      end
    end

    def destroy_user_non_working_calendar_days
      calendar_month.calendar_days.where(kind: CalendarDay::KINDS[:user_non_working]).destroy_all
      Success()
    end

    def generate_calendar_days(calendar_month)
      User.inspector.ordered.where(project_type: calendar_month.project_type).each.with_index do |inspector, index|
        pattern = index % 2
        days = generate_working_days(inspector, calendar_month, pattern)
        days.each do |day|
          CalendarDay.find_or_create_by!(user: inspector, kind: :working, date: day[:date], calendar_month: calendar_month) do |calendar_day|
            calendar_day.working_hours = day[:working_hours]
          end
        end
      end
      Success()
    end

    def generate_working_days(inspector, calendar_month, pattern) # rubocop:disable Metrics/MethodLength, Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
      working_hours_norms = calculate_inspector_working_hours_norms(inspector)
      days = []
      sum_of_hours = 0

      inspector_vacations = calendar_month.calendar_days.where(kind: CalendarDay::KINDS.values_at(:vacation), user: inspector)

      (available_days.to_a - calendar_month.non_working_days.pluck(:date) - inspector_vacations.pluck(:date))
        .select { |date| date.wday.in?(PATTERNS[pattern]) }.each do |date|
        hours = date.saturday? ? 8.5 : WORKING_HOURS
        sum_of_hours += hours
        days << { date: date, working_hours: hours }
      end

      random_days = days.select { |day| day[:date].wday.in?([2, 5]) } # select Tuesday and Thursday, because it is more common
      should_be_deleted_days_count = ((sum_of_hours - working_hours_norms) / WORKING_HOURS).ceil

      should_be_deleted_days_count.times do
        random_day = random_days.sample
        if sum_of_hours - working_hours_norms >= WORKING_HOURS
          random_days = random_days.delete_if { |day| day[:date] == random_day[:date] }
          days = days.delete_if { |day| day[:date] == random_day[:date] }
          sum_of_hours -= WORKING_HOURS
        else
          hours = WORKING_HOURS - (sum_of_hours - working_hours_norms)
          days = days.delete_if { |day| day[:date] == random_day[:date] }
          days << { date: random_day[:date], working_hours: hours }
          sum_of_hours -= hours
        end
      end

      days
    end

    def calculate_inspector_working_hours_norms(inspector)
      vacation_days_count = calendar_month.calendar_days.where(kind: CalendarDay::KINDS.values_at(:vacation), user: inspector).count
      days_in_month = calendar_month.date.all_month.count
      ((1 - vacation_days_count.to_f / days_in_month) * calendar_month.working_norm.hours).ceil
    end

    def start_work_schedule_generation(calendar_month)
      calendar_month.start_work_schedule_generation!
      Success()
    rescue AASM::InvalidTransition
      Failure(message: [I18n.t('errors.schedule.cannot_create')])
    end

    def finish_work_schedule_generation(calendar_month)
      calendar_month.finish_work_schedule_generation!
      Success()
    end
  end
end
