# frozen_string_literal: true

module WorkSchedules
  class Update
    include Dry::Monads[:result, :do]
    include AfterCommitEverywhere
    include Schedulable

    static_facade :call
    pattr_initialize :calendar_month, :params, :current_user

    def call
      lock(calendar_month, ttl: 1.minute, error_messages: [I18n.t('errors.schedule.cannot_update')]) do
        ActiveRecord::Base.transaction do
          calendar_days = yield upsert_calendar_days

          after_commit do
            broadcast_to_subscribers(action: :calendar_days_update, calendar_month: calendar_month, calendar_days: calendar_days)
          end
        end
      end
      Success()
    end

    private

    def upsert_calendar_days # rubocop:disable Metrics/MethodLength
      calendar_day_ids = sanitized_calendar_days_params.map do |attributes|
        calendar_day = CalendarDay.find_or_initialize_by(date: attributes[:date], user_id: attributes[:user_id], calendar_month: calendar_month)
        next if calendar_day.inspections.where.not(state: :canceled).exists? || calendar_day.vacation?

        calendar_day.update(
          kind: attributes[:kind],
          working_hours: attributes[:working_hours]
        )
        calendar_day.id
      end.compact
      calendar_days = CalendarDay.where(id: calendar_day_ids)
      Success(calendar_days)
    end

    def sanitized_calendar_days_params # rubocop:disable Metrics/MethodLength, Metrics/AbcSize
      user_ids = params['calendar_days'].map { |calendar_day| calendar_day['user_id'] }
      scope = if current_user.superadmin?
                User.where(id: user_ids, project_type: calendar_month.project_type)
              elsif current_user.admin?
                User.where(id: user_ids, project_type: current_user.project_type)
              elsif current_user.curator?
                User.where(id: user_ids, project_type: current_user.project_type, curator_id: current_user.id)
              else
                raise
              end
      managable_user_ids = scope.ids
      params['calendar_days'].select { |calendar_day| calendar_day['user_id'].in?(managable_user_ids) }
    end

    def broadcast_to_subscribers(action:, calendar_month:, calendar_days:) # rubocop:disable Metrics/MethodLength
      ActionCable.server.broadcast(
        "admin:schedule:#{calendar_month.project_type}",
        action: action,
        date: calendar_month.date,
        calendarDays: CamelizedPartialRenderer.call(
          partial: 'api/admin/calendar_days/ws_calendar_days',
          locals: { calendar_days: calendar_days }
        )
      )

      calendar_days.joins(user: :curator).includes(:user).group_by { |c| c.user.curator_id }.each do |curator_id, grouped_calendar_days|
        ActionCable.server.broadcast(
          "admin:schedule:#{calendar_month.project_type}:#{curator_id}",
          action: action,
          date: calendar_month.date,
          calendarDays: CamelizedPartialRenderer.call(
            partial: 'api/admin/calendar_days/ws_calendar_days',
            locals: { calendar_days: grouped_calendar_days }
          )
        )
      end
    end
  end
end
