# frozen_string_literal: true

class BaseSearcher # rubocop:disable Metrics/ClassLength
  ALLOWED_SORTING_VALUES = %w[desc asc].freeze
  Boolean = Class.new

  class << self
    attr_reader :model_name

    def includes(*associations)
      @includes ||= associations
    end

    def search_fields(*fields)
      @search_fields ||= fields
    end

    def allowed_filters
      @allowed_filters ||= ActiveSupport::HashWithIndifferentAccess.new
    end

    def allowed_sortings
      @allowed_sortings ||= ActiveSupport::HashWithIndifferentAccess.new
    end

    def filter(name, type, options = {})
      allowed_filters[name] = type == Hash ? [ActiveSupport::HashWithIndifferentAccess, options] : [type, options]
    end

    def sorting(name, options = {})
      allowed_sortings[name] = options
    end

    def model(class_name)
      @model_name = class_name
    end

    def call(params = ActionController::Parameters.new, current_user = nil, current_project_type = nil)
      new(params, current_user, current_project_type).call
    end
  end

  attr_reader :params
  attr_accessor :where, :order, :current_user, :current_project_type, :boost_where, :applied_filters

  def initialize(params, current_user, current_project_type)
    @params = params.is_a?(ActionController::Parameters) ? params.permit!.to_h : HashWithIndifferentAccess.new(params)
    @where = {}
    @boost_where = {}
    @order = []
    @current_user = current_user
    @current_project_type = current_project_type
    @applied_filters = []
  end

  private_class_method :new

  def call # rubocop:disable Metrics/MethodLength
    before_filter
    build_where_query
    after_filter
    build_order_query if params[:search].blank?
    apply_base_scope

    class_name.search(term, page: params[:page],
                            per_page: params[:per_page] || Pagy::VARS[:items],
                            where: where,
                            boost_where: boost_where,
                            order: order,
                            includes: self.class.includes,
                            fields: self.class.search_fields.presence)
  end

  private

  def class_name
    self.class.model_name || self.class.to_s.sub('Searcher', '').constantize
  end

  def term
    @term = params[:search].presence || '*'
  end

  def before_filter; end

  def after_filter; end

  def apply_base_scope; end

  def build_where_query
    return unless params[:filters].is_a?(Array)

    params[:filters].each do |filter|
      next unless allowed_filter?(filter)

      parse_filter(filter)
      applied_filters << filter[:field]
    end
  end

  def build_order_query
    return self.order = default_order unless params[:sortings].is_a?(Array)

    params[:sortings].each do |sorting|
      next unless allowed_sorting?(sorting)

      field = sorting[:field].to_s
      field += "_#{I18n.locale}" if self.class.allowed_sortings[sorting[:field]][:localize] && I18n.locale != :en
      order << { field => { order: sorting[:value], missing: '_last', unmapped_type: :long } } if allowed_sorting?(sorting)
    end
  end

  def default_order
    [{ id: { order: 'asc', missing: '_last', unmapped_type: :long } }]
  end

  def parse_filter(filter) # rubocop:disable Metrics/AbcSize
    if as(filter).present?
      where[as(filter)] = filter[:value]
    elsif custom_method_for(filter).present?
      send(custom_method_for(filter), cast_value(filter[:value] || filter[:range]))
    elsif filter[:value].present?
      where[filter[:field]] = filter[:value]
    elsif filter[:range].present?
      where[filter[:field]] = filter[:range].to_h.delete_if { |_, value| value.blank? }.symbolize_keys
    end
  end

  def as(filter)
    self.class.allowed_filters[filter[:field]].second[:as]
  end

  def custom_method_for(filter)
    self.class.allowed_filters[filter[:field]].second[:method]
  end

  def allowed_filter?(filter)
    self.class.allowed_filters.key?(filter[:field]) &&
      self.class.allowed_filters[filter[:field]].first == get_class_from(filter[:value] || filter[:range])
  end

  def allowed_sorting?(sorting)
    self.class.allowed_sortings.key?(sorting[:field]) && ALLOWED_SORTING_VALUES.include?(sorting[:value])
  end

  def get_class_from(value)
    casted_value = cast_value(value)
    if casted_value.is_a?(TrueClass) || casted_value.is_a?(FalseClass)
      Boolean
    else
      value.class
    end
  end

  def cast_value(value) # rubocop:disable Metrics/MethodLength
    return value unless value.is_a?(String)

    case value
    when /\A(true|t)\z/i
      true
    when /\A(false|f)\z/i
      false
    when /\A(nil|null)\z/i
      nil
    when /\A\d+\z/
      value.to_i
    when /\A\d*\.\d+\z/
      value.to_f
    else
      value
    end
  end

  def method_missing(method_name, *_args)
    if respond_to_missing?(method_name)
      filter_name = method_name.to_s.gsub('_filter_applied?', '')
      applied_filters.include?(filter_name)
    else
      super
    end
  end

  def respond_to_missing?(method_name)
    filter_name = method_name.to_s.gsub('_filter_applied?', '')
    self.class.allowed_filters.key?(filter_name) || super
  end
end
