# frozen_string_literal: true

class GenerateReportMatrix
  def self.call(date, project_type)
    range = date.beginning_of_month.beginning_of_week..date.end_of_month.end_of_week # bump to whole weeks

    inspections =
      Inspection
      .finished
      .includes(:inspection_results)
      .where(inspection_results: { inspection_id: nil }) # лишний раз не дергать те, по которым уже сгенерены результаты (например для другого отчета)
      .where(inspectable_type: project_type.camelize)
      .where(date: range)

    GenerateInspectionResults.call(inspections)

    inspection_results =
      InspectionResult
      .failed
      .select('COUNT(inspection_results.parameter_id) AS total_count, inspection_results.parameter_id, inspections.date AS date')
      .left_joins(:inspection)
      .where(inspections: { state: :finished, date: [range], inspectable_type: project_type.camelize })
      .includes(:parameter)
      .group('inspection_results.parameter_id, inspections.date')
      .order('date ASC, parameter_id ASC')

    # неоптимально стопицот раз дергаем Parameter.find
    parameter_index = inspection_results.map { |r| r.parameter.id }.uniq.map.with_index { |id, index| [id, index] }.to_h
    row_count = parameter_index.size

    result_matrix = generate_result_matrix_for(inspection_results, range, parameter_index)
    weekly_summary_matrix = generate_weekly_summary_matrix_for(result_matrix, range, row_count)

    ReportMatrix.new(range, parameter_index, result_matrix, weekly_summary_matrix)
  end

  def self.generate_result_matrix_for(inspection_results, date_range, parameter_index)
    row_count = parameter_index.size
    date_array = date_range.to_a
    column_count = date_array.size

    result_matrix = Matrix.zero(row_count, column_count)

    # заполнить матрицу каунтами нарушений
    inspection_results.each { |result| result_matrix[parameter_index[result.parameter.id] - 1, date_array.index(result.date)] = result.total_count }

    result_matrix
  end

  def self.generate_weekly_summary_matrix_for(result_matrix, date_range, row_count)
    col_count = date_range.count(&:sunday?)
    summary_matrix = Matrix.zero(row_count, col_count)

    result_matrix.to_a.each.with_index do |row, row_index|
      row.each_slice(7).with_index do |week, col_index|
        summary_matrix[row_index, col_index] = week.sum
      end
    end

    summary_matrix
  end
end
