# frozen_string_literal: true

require 'sidekiq/web'

Rails.application.routes.draw do
  mount Rswag::Ui::Engine => '/api-docs'
  mount Rswag::Api::Engine => '/api-docs'

  if Rails.env.production?
    Sidekiq::Web.use Rack::Auth::Basic do |username, password|
      ActiveSupport::SecurityUtils.secure_compare(::Digest::SHA256.hexdigest(username), ::Digest::SHA256.hexdigest(ENV['SIDEKIQ_USERNAME'])) &
        ActiveSupport::SecurityUtils.secure_compare(::Digest::SHA256.hexdigest(password), ::Digest::SHA256.hexdigest(ENV['SIDEKIQ_PASSWORD']))
    end
  end

  namespace :api, defaults: { format: :json } do
    mount Sidekiq::Web => '/sidekiq'

    get    'checklist/:project', to: 'checklists#show'
    put    'checklist/:project', to: 'checklists#update'
    delete 'checklist/:project', to: 'checklists#destroy'

    resources :inspections, only: %i[index show update] do
      member do
        post :start
        post :finish
        post :attach_file
        delete :purge_file
      end
    end
    resources :calendar_days, only: %i[index]
    resources :codebooks, only: %i[index]
    resources :direct_uploads, only: %i[create]
    resources :criteria, only: %i[index]

    namespace :admin do
      resources :policlinics, only: %i[index show create update destroy]
      resources :mfcs,        only: %i[index show create update destroy]
      resources :users,       only: %i[index show create update destroy] do
        put :reactivate, on: :member
        put :mass_update, on: :collection
      end
      resources :non_working_days, only: %i[index show create destroy]
      resources :work_schedules, only: %i[show create update destroy], param: :month
      resources :inspection_schedules, only: %i[show create destroy], param: :month
      resources :object_schedules, only: %i[show], param: :month
      resources :vacations, only: %i[index show create update destroy]
      resources :inspections, only: %i[index show create update] do
        put :mass_destroy, on: :collection
        put :cancel,       on: :collection
      end
      resources :checklists
      resources :repair_periods, only: %i[create update destroy]
      resources :criteria do
        put :update_position, on: :member
      end
      resources :working_norms
      resources :reports, only: %i[index show create]
      resources :calendar_days, only: %i[update]

      put '/project_types/set'

      resources :codebooks, only: [] do
        collection do
          get :administrative_districts
          get :curators
          get :territories
          get :project_types
          get :roles
          get :states
          get :policlinic_kinds
          get :policlinic_sizes
          get :mfc_divisions
          get :mfc_kinds
          get :districts
          get :policlinics
          get :mfcs
          get :checklists
          get :inspection_states
          get :inspection_cancel_reasons
          get :report_kinds
          get :calendar_day_absence_reasons
          get 'checklist_parameters/:checklist_id', action: :checklist_parameters
        end
      end

      resources :statistics, only: [] do
        collection do
          get 'inspections/:month', action: :inspections
          get 'inspectors/:month', action: :inspectors
          get 'working_inspectors/:month', action: :working_inspectors
        end
      end

      resources :file_archives, only: %i[index show] do
        get :download, on: :member
      end
    end

    namespace :auth do
      resources :sessions, only: [] do
        collection do
          post 'refresh'
          post 'sign_in', action: :create
          delete 'sign_out', action: :destroy
        end
      end
      resources :passwords, only: [] do
        collection do
          post 'forgot'
          post 'check_token_expiration'
          post 'reset'
          post 'send_new_link'
        end
      end
    end
  end
end
