# frozen_string_literal: true

require 'rails_helper'

RSpec.describe User, type: :model do
  it_behaves_like 'ValidFactory'

  it { is_expected.to have_secure_password }

  it { is_expected.to define_enum_for(:role).with_values(User::ROLES).backed_by_column_of_type(:string) }
  it { is_expected.to define_enum_for(:gender).with_values(User::GENDERS).backed_by_column_of_type(:string) }
  it { is_expected.to define_enum_for(:project_type).with_values(ProjectTypes::ALLOWED_TYPES).backed_by_column_of_type(:string) }

  describe 'associations' do
    it { is_expected.to belong_to(:curator).optional }

    it { is_expected.to have_many(:inspectors) }
    it { is_expected.to have_many(:calendar_days) }
    it { is_expected.to have_many(:vacations) }
  end

  describe 'validations' do
    subject { create(:user) }

    it { is_expected.to validate_presence_of(:email) }
    it { is_expected.to validate_presence_of(:first_name) }
    it { is_expected.to validate_presence_of(:last_name) }
    it { is_expected.to validate_presence_of(:role) }
    it { is_expected.to validate_presence_of(:project_type) }

    it { is_expected.to allow_value('vasya@pupkin.com').for(:email) }
    it { is_expected.not_to allow_value('invalid').for(:email) }

    it { is_expected.to validate_length_of(:first_name).is_at_most(50) }
    it { is_expected.to validate_length_of(:middle_name).is_at_most(50) }
    it { is_expected.to validate_length_of(:last_name).is_at_most(50) }
    it { is_expected.to validate_length_of(:recovery_password_token).is_at_least(User::MIN_RECOVERY_TOKEN_LENGTH) }

    it { is_expected.to validate_uniqueness_of(:email).ignoring_case_sensitivity }
    it { is_expected.to validate_uniqueness_of(:phone).ignoring_case_sensitivity }
    it { is_expected.to validate_uniqueness_of(:recovery_password_token).ignoring_case_sensitivity }
  end

  describe 'callbacks' do
    it 'strips whitespaces and converts email to lowercase' do
      user = build(:user, email: ' VASya@PuPkin.cOM ')
      user.save
      expect(user.email).to eq 'vasya@pupkin.com'
    end

    it 'strips whitespaces in email' do
      user = build(:user, email: ' vasya@pupkin.com ')
      user.save
      expect(user.email).to eq 'vasya@pupkin.com'
    end

    it 'converts email to lowercase' do
      user = build(:user, email: 'VASya@PuPkin.cOM')
      user.save
      expect(user.email).to eq 'vasya@pupkin.com'
    end
  end

  describe '#generate_recovery_password_link' do
    context 'when duplicate token' do
      let(:recovery_password_token1) { 'aef617dfc26d82fe1914bb87dd26cf3dbfcc2f5c281f2234109357ffe0cdbb03' }
      let(:recovery_password_token2) { '111e0869d0240eda326bafab6dcd11a45e85b128b87a169ce3bf756e8d03743e' }
      let(:user) { create(:user) }

      before do
        create(:user, recovery_password_token: recovery_password_token1)
        allow(SecureRandom).to receive(:hex).and_return(recovery_password_token1, recovery_password_token2)
      end

      it 'safely regenerates recovery_password_link' do
        link = user.generate_recovery_password_link
        expect(link).to eq("#{ENV.fetch('FRONTEND_URL')}/auth/recovery-password?token=#{recovery_password_token2}")
      end
    end

    context 'when uniq token' do
      let(:recovery_password_token) { 'aef617dfc26d82fe1914bb87dd26cf3dbfcc2f5c281f2234109357ffe0cdbb03' }
      let(:user) { create(:user) }

      before { allow(SecureRandom).to receive(:hex).and_return(recovery_password_token) }

      it 'generates recovery_password_link' do
        link = user.generate_recovery_password_link
        expect(link).to eq("#{ENV.fetch('FRONTEND_URL')}/auth/recovery-password?token=#{recovery_password_token}")
      end
    end
  end
end
