# frozen_string_literal: true

require 'rails_helper'

RSpec.describe InspectionSchedules::Clean do
  describe '.call' do
    subject(:operation) { described_class.call(calendar_month) }

    context 'with valid params' do
      before { Timecop.freeze(Date.current.end_of_month - 15.days) }

      after { Timecop.return }

      let(:calendar_month) { create(:calendar_month, date: Date.current) }
      let!(:calendar_day) { create(:calendar_day, :working, calendar_month: calendar_month, date: 1.day.from_now) }
      let!(:inspection) { create(:inspection, calendar_day: calendar_day) }

      it { is_expected.to be_success }

      it 'deletes planned inspections for given month' do
        expect { operation }.to change(Inspection.planned, :count).by(-1)
      end
    end

    context 'when given month has already passed' do
      let(:calendar_month) { create(:calendar_month, date: 2.months.ago) }
      let!(:inspection) { create(:inspection, calendar_month: calendar_month, date: 2.months.ago) }

      it { is_expected.to be_failure }

      it 'doesnt delete any inspections and returns error message' do
        expect { operation }.not_to change(Inspection, :count)
        expect(operation.failure[:message]).to include(I18n.t('errors.schedule.cannot_clean_for_passed_month'))
      end
    end

    context 'when cleaning in the middle of the month' do
      before { Timecop.freeze(Date.current.end_of_month - 15.days) }

      after { Timecop.return }

      let(:calendar_month) { create(:calendar_month, date: Date.current) }
      let!(:passed_calendar_day) { create(:calendar_day, kind: :working, date: 1.day.ago, calendar_month: calendar_month) }
      let!(:future_calendar_day) { create(:calendar_day, kind: :working, date: 1.day.from_now, calendar_month: calendar_month) }
      let!(:passed_inspection) { create(:inspection, calendar_day: passed_calendar_day) }
      let!(:future_inspection) { create(:inspection, calendar_day: future_calendar_day) }

      it { is_expected.to be_success }

      it 'deletes future inspections' do
        operation

        expect { future_inspection.reload }.to raise_error(ActiveRecord::RecordNotFound)
      end

      it 'does not delete passed inspections' do
        operation

        expect { passed_inspection.reload }.not_to raise_error
      end
    end
  end
end
