# frozen_string_literal: true

require 'rails_helper'

RSpec.describe Inspections::Update do
  describe '.call' do
    subject(:operation) { described_class.call(inspection, params, superadmin) }

    let_it_be(:date) { Date.current }
    let_it_be(:admin) { create(:user, :admin, project_type: 'policlinic') }
    let_it_be(:superadmin) { create(:user, :superadmin) }
    let_it_be(:curator) { create(:user, :curator) }
    let_it_be(:inspector) { create(:user, project_type: 'mfc', curator: curator) }
    let_it_be(:calendar_day) { create(:calendar_day, date: date, user: inspector) }
    let_it_be(:inspectable) { create(:mfc) }
    let_it_be(:inspectable_with_repair_period) { create(:mfc, :with_repair_period) }
    let_it_be(:inspectable_type) { 'mfc' }
    let_it_be(:checklist) { create(:checklist, project_type: 'mfc') }
    let_it_be(:inspection)  { create(:inspection, ends_at: 1.year.from_now) }

    let(:params) do
      ActionController::Parameters.new(
        date: date,
        starts_at: Time.current + 1.1.days,
        inspectable_id: inspectable.id,
        inspectable_type: inspectable_type,
        checklist_id: checklist.id,
        inspector_id: inspector.id
      )
    end

    let(:params_with_repair_overlaping) do
      ActionController::Parameters.new(
        date: date,
        starts_at: Time.current + 5.days,
        inspectable_id: inspectable_with_repair_period.id,
        inspectable_type: 'mfc',
        checklist_id: checklist.id,
        inspector_id: inspector.id
      )
    end

    context 'with valid params' do
      it { is_expected.to be_success }

      it 'updates an inspection' do
        operation
        expect(inspection.date).to eq params[:date]
        expect(inspection.starts_at).to eq params[:starts_at]
        expect(inspection.inspectable).to eq inspectable
        expect(inspection.checklist).to eq checklist
        expect(inspection.inspector).to eq inspector
      end
    end

    context 'when inspectable repair period overlapping inspection date' do
      subject(:operation) { described_class.call(inspection, params_with_repair_overlaping, superadmin) }

      it { is_expected.to be_failure }

      it 'returns error message' do
        expect(operation.failure[:message]).to include(I18n.t('errors.cant_create.inspection.because_overlapping_repair_period'))
      end
    end

    context 'when admin sets checklist from another project' do
      subject(:operation) { described_class.call(inspection, params, admin) }

      it { is_expected.to be_failure }

      it 'returns error message' do
        expect(operation.failure[:message]).to include(I18n.t('errors.cant_update.inspection.for_another_project'))
      end
    end

    context 'when the curator appoints a specialist who does not supervise' do
      subject(:operation) { described_class.call(inspection, params, curator) }

      let(:inspector) { create(:user, project_type: 'mfc') }

      it { is_expected.to be_failure }

      it 'returns error message' do
        expect(operation.failure[:message]).to include(I18n.t('errors.inspection.cannot_set_inspector'))
      end
    end

    context 'when Checklist project_type doesnt match with Inspection project_type' do
      subject(:operation) { described_class.call(inspection, params, superadmin) }

      let(:checklist) { create(:checklist, project_type: 'policlinic') }

      it { is_expected.to be_failure }

      it 'returns error message' do
        expect(operation.failure[:checklist_id]).to include(I18n.t('errors.cant_create.inspection.project_type_mismatch'))
      end
    end
  end
end
