# frozen_string_literal: true

require 'rails_helper'

describe Users::Create do
  subject { described_class.call(params, current_user) }

  context 'when current_user is superadmin' do
    let(:params) { attributes_for(:user) }
    let(:current_user) { build_stubbed(:user, :superadmin) }

    it { is_expected.to be_success }

    it 'creates a user' do
      expect(Auth::PasswordsMailer).to receive(:invite).and_call_original
      expect { described_class.call(params, current_user) }
        .to(change(User, :count).by(1))
    end
  end

  context 'when current_user is admin of targer user project' do
    let(:params) { attributes_for(:user, project_type: 'mfc') }
    let(:current_user) { build_stubbed(:user, :admin, project_type: 'mfc') }

    it { is_expected.to be_success }

    it 'creates a user' do
      expect { described_class.call(params, current_user) }
        .to(change(User, :count).by(1))
    end

    context 'when trying to create overpowered user' do
      let(:params) { attributes_for(:user, project_type: 'mfc', role: 'superadmin') }

      it { is_expected.to be_failure }

      it 'does not create a user' do
        expect { described_class.call(params, current_user) }
          .not_to(change(User, :count))
      end
    end

    context 'when reinvite archived user' do
      let!(:archived_user) { create(:user, :mfc, :archived) }
      let!(:another_archived_user) { create(:user, :mfc, :archived) }
      let(:params) { attributes_for(:user, project_type: 'mfc').merge(email: archived_user.email) }
      let(:params2) { attributes_for(:user, project_type: 'mfc').merge(email: another_archived_user.email) }

      it 'creates a user' do
        expect(Auth::PasswordsMailer).to receive(:invite).and_call_original
        expect { described_class.call(params, current_user) }
          .to(change(User, :count).by(1))
        expect { archived_user.reload }.to change(archived_user, :email).to(nil)

        expect(Auth::PasswordsMailer).to receive(:invite).and_call_original
        expect { described_class.call(params2, current_user) }
          .to(change(User, :count).by(1))
        expect { another_archived_user.reload }.to change(another_archived_user, :email).to(nil)
      end
    end
  end

  context 'when current_user is admin of different project' do
    let(:params) { attributes_for(:user, project_type: 'mfc') }
    let(:current_user) { build_stubbed(:user, :admin, project_type: 'policlinic') }

    it { is_expected.to be_failure }

    it 'does not create a user' do
      expect { described_class.call(params, current_user) }
        .not_to(change(User, :count))
    end
  end
end
