# frozen_string_literal: true

require 'rails_helper'

describe Users::Update do
  subject { described_class.call(user, params, current_user) }

  context 'when current_user is superadmin' do
    let(:user) { create(:user, :policlinic) }
    let(:params) { attributes_for(:user).merge(first_name: user.first_name + '1') }
    let(:current_user)  { build(:user, :superadmin) }

    it { is_expected.to be_success }

    it 'updates a user' do
      expect { described_class.call(user, params, current_user) }
        .to(change(user, :first_name))
    end

    context 'and updates a user email' do
      let(:invited_user) { create(:user, :policlinic, state: 'invited') }
      let(:registered_user) { create(:user, :policlinic, state: 'registered') }
      let(:params) { attributes_for(:user).merge(email: user.email + 'ru') }

      it 'resends invite email if user invited' do
        expect(Auth::PasswordsMailer).to receive(:invite).with(invited_user).and_call_original
        expect { described_class.call(invited_user, params, current_user) }
          .to(change(invited_user, :email))
      end

      it 'does not resend invite email if user registered' do
        expect(Auth::PasswordsMailer).not_to receive(:invite).with(registered_user).and_call_original
        expect { described_class.call(registered_user, params, current_user) }
          .to(change(registered_user, :email))
      end
    end

    it 'allows updating to any role' do
      %w[superadmin admin curator inspector].each do |role|
        expect { described_class.call(user, { role: role }, current_user) }
          .to(change(user, :role).to(role))
      end
    end

    it 'allows change of project_type' do
      expect { described_class.call(user, { project_type: 'mfc' }, current_user) }
        .to(change(user, :project_type).to('mfc'))
    end
  end

  context 'when current_user is admin' do
    let(:user) { create(:user, :inspector, :policlinic) }
    let(:current_user) { build(:user, :admin, :policlinic) }

    it 'allows updating user role to only curator and inspector' do
      %w[curator inspector].each do |role|
        expect { described_class.call(user, { role: role }, current_user) }
          .to(change(user, :role).to(role))
      end
      %w[admin superadmin].each do |disallowed_role|
        expect { described_class.call(user, { role: disallowed_role }, current_user) }
          .not_to(change(user, :role))
      end
    end

    it 'does not allow change of project_type' do
      expect { described_class.call(user, { project_type: 'mfc' }, current_user) }
        .not_to(change(user, :project_type))
    end
  end

  context 'when current_user is curator' do
    let(:current_user) { create(:user, :curator, :policlinic) }
    let(:user) { create(:user, :inspector, :policlinic, curator: current_user) }

    it 'does not allow escalation of role' do
      %w[admin superadmin inspector curator].each do |role|
        expect { described_class.call(user, { role: role }, current_user) }
          .not_to(change(user, :role))
      end
    end

    it 'does not allow change of project_type' do
      expect { described_class.call(user, { project_type: 'mfc' }, current_user) }
        .not_to(change(user, :project_type))
    end
  end
end
