# frozen_string_literal: true

require 'rails_helper'

describe Vacations::Create do
  subject { described_class.call(current_user, user, params) }

  let(:params) { {} }
  let(:current_user) { create(:user, :superadmin) }

  context 'when current user is not an admin of user project' do
    let(:current_user) { create(:user, :admin, project_type: 'policlinic') }
    let(:user) { create(:user, :inspector, project_type: 'mfc') }

    it { is_expected.to be_failure }
  end

  context 'when current user is an admin of user project' do
    let(:current_user) { create(:user, :admin, project_type: 'mfc') }
    let(:user) { create(:user, :inspector, project_type: 'mfc') }
    let(:params) { attributes_for(:vacation).merge(user_id: user.id) }

    it { is_expected.to be_success }
  end

  context 'when current user is not a curator of user' do
    let(:current_user) { create(:user, :curator, project_type: 'mfc') }
    let(:user) { create(:user, :inspector, :with_curator, project_type: 'mfc') }

    it { is_expected.to be_failure }
  end

  context 'when current user is a curator of user' do
    let(:current_user) { create(:user, :curator, project_type: 'mfc') }
    let(:user) { create(:user, :inspector, project_type: 'mfc', curator_id: current_user.id) }
    let(:params) { attributes_for(:vacation).merge(user_id: user.id) }

    it { is_expected.to be_success }
  end

  context 'when worker has planned inspections' do
    let(:user) { create(:user, :inspector, :with_planned_inspections) }

    it { is_expected.to be_failure }
  end

  context 'when worker has overlapping vacations' do
    let(:user) { create(:user, :inspector) }
    let!(:vacation) { create(:vacation, user: user) }
    let(:params) { vacation.attributes.slice('starts_at', 'ends_at') }

    it { is_expected.to be_failure }
  end

  context 'when valid params' do
    let(:user) { create(:user, :inspector) }
    let(:params) { attributes_for(:vacation).merge(user_id: user.id) }

    it { is_expected.to be_success }

    it 'creates vacation and calendar days' do
      expect { described_class.call(current_user, user, params) }
        .to change(user.vacations, :count).by(1)
                                          .and change(user.calendar_days.vacation, :count)
    end
  end
end
