# frozen_string_literal: true

require 'rails_helper'

describe Vacations::Update do
  let_it_be(:vacation) { create(:vacation) }
  let_it_be(:user) { vacation.user }

  subject { described_class.call(user, vacation, params.with_indifferent_access) }

  context 'when target user has planned inspections' do
    let(:user) { create(:user, :inspector, :with_planned_inspections) }
    let(:params) { { user_id: user.id } }

    it { is_expected.to be_failure }
  end

  context 'when target user has overlapping vacations' do
    let!(:vacation2) { create(:vacation, user: user, starts_at: vacation.starts_at + 10.days, ends_at: vacation.ends_at + 10.days) }
    let(:params) { vacation2.attributes.slice('starts_at', 'ends_at').merge(user_id: user.id) }

    it { is_expected.to be_failure }
  end

  context 'when valid params' do
    let(:params) { attributes_for(:vacation, starts_at: 20.days.from_now, ends_at: 30.days.from_now).merge(user_id: user.id) }

    it 'updates the vacation' do
      expect { described_class.call(user, vacation, params.with_indifferent_access) }
        .to change(vacation, :starts_at)
        .and change(vacation, :ends_at)
        .and change(vacation.calendar_days, :count).from(3).to(11)
    end
  end

  context 'when user changed' do
    let!(:user) { create(:user) }
    let(:params) { { user_id: user.id } }
    let!(:vacation) { create(:vacation) }

    it 'updates the user and reassigns the calendar day to new user' do
      expect { described_class.call(user, vacation, params.with_indifferent_access) }
        .to(change(vacation, :user_id)
        .and(change { vacation.calendar_days.map(&:user_id) }))
    end
  end
end
