# frozen_string_literal: true

require 'swagger_helper'

describe 'ADMIN Checklists API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_CHECKLISTS_TAG = 'Admin Checklists'

  sign_in_superadmin

  let_it_be(:checklists) { create_list(:checklist, 2) }
  let_it_be(:checklist) { create(:checklist) }

  before { Checklist.reindex }

  path '/checklists' do
    get 'Gets checklists' do
      include_context 'RequestDefaultParams', ADMIN_CHECKLISTS_TAG
      include_context 'SearchDefaultParams', ChecklistSearcher

      it_behaves_like 'API Paginatable', :checklist

      response 200, 'Checklists found' do
        schema '$ref' => 'swagger/schemas/admin/checklists/index.json'

        run_test!
      end
    end

    post 'Creates a checklist' do
      include_context 'RequestDefaultParams', ADMIN_CHECKLISTS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[projectType rawData name],
        properties: {
          projectType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values },
          raw_data: { type: :string },
          name: { type: :string }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Unprocessable Entity',
                      as: 'Checklist',
                      with_params: { project_type: '' },
                      on: :create

      response 201, 'Checklist created' do
        let(:params) { attributes_for(:checklist) }

        schema '$ref' => 'swagger/schemas/admin/checklists/show.json'

        it 'creates a checklist' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Checklist, :count).by(1))

          created = Checklist.find(json_response[:id])

          params.keys.each do |attribute|
            expect(created.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/checklists/{id}' do
    parameter name: :id, in: :path, type: :integer

    get 'Gets a checklist by ID' do
      include_context 'RequestDefaultParams', ADMIN_CHECKLISTS_TAG

      it_behaves_like 'Not Found', resource_name: 'Checklist'
      let(:id) { create(:checklist).id }

      response 200, 'Checklist found' do
        schema '$ref' => 'swagger/schemas/admin/checklists/show.json'
        run_test!
      end
    end

    put 'Updates a checklist' do
      include_context 'RequestDefaultParams', ADMIN_CHECKLISTS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          properties: {
            projectType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values },
            raw_data: { type: :string },
            name: { type: :string }
          }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Checklist'
      it_behaves_like 'Unprocessable Entity',
                      as: 'Checklist',
                      with_params: { project_type: '' },
                      on: :update

      response 200, 'Checklist updated' do
        let!(:id) { checklist.id }
        let(:params) { attributes_for(:checklist) }

        schema '$ref' => 'swagger/schemas/admin/checklists/show.json'

        it 'updates a checklist' do |example|
          submit_request(example.metadata)

          updated = Checklist.find(json_response[:id])
          params.keys.each do |attribute|
            expect(updated.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end

    delete 'Deletes a checklist' do
      include_context 'RequestDefaultParams', ADMIN_CHECKLISTS_TAG, true

      it_behaves_like 'Not Found', resource_name: 'Checklist'

      response 204, 'Checklist deleted' do
        let!(:id) { create(:checklist).id }

        it 'deletes a checklist' do |example|
          expect { submit_request(example.metadata) }
            .to change(Checklist, :count).by(-1)

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end
end
