# frozen_string_literal: true

require 'swagger_helper'

describe 'Admin Criteria API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_CRITERIA_TAG = 'Admin Criteria'

  sign_in_superadmin

  let_it_be(:criteria) { create_list(:criterion, 2) }
  let_it_be(:criterion) { create(:criterion, :select_with_options) }

  before { Criterion.reindex }

  path '/criteria' do
    get 'Gets available Criteria' do
      include_context 'RequestDefaultParams', ADMIN_CRITERIA_TAG
      include_context 'SearchDefaultParams', CriterionSearcher

      response 200, 'Criteria found' do
        schema '$ref' => 'swagger/schemas/admin/criteria/index.json'
        run_test!
      end
    end

    post 'Creates a Criterion' do
      include_context 'RequestDefaultParams', ADMIN_CRITERIA_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[title projectType inputType],
        properties: {
          title: { type: :string },
          projectType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values },
          inputType: { type: :string, enum: Criterion::INPUT_TYPES },
          selectOptions: { type: :array, items: { type: :string } }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Unprocessable Entity',
                      as: 'Criterion',
                      with_params: { project_type: '' },
                      on: :create

      response 201, 'Criterion created' do
        let(:params) { attributes_for(:criterion, :select_with_options) }

        schema '$ref' => 'swagger/schemas/admin/criteria/show.json'

        it 'creates a criterion' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Criterion, :count).by(1))

          criterion = Criterion.find(json_response[:id])

          params.keys.each do |attribute|
            expect(criterion.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/criteria/{id}' do
    parameter name: :id, in: :path, type: :integer

    get 'Gets a Criterion by ID' do
      include_context 'RequestDefaultParams', ADMIN_CRITERIA_TAG

      it_behaves_like 'Not Found', resource_name: 'Criterion'
      let(:id) { create(:criterion).id }

      response 200, 'Criterion found' do
        schema '$ref' => 'swagger/schemas/admin/criteria/show.json'
        run_test!
      end
    end

    put 'Updates a Criterion' do
      include_context 'RequestDefaultParams', ADMIN_CRITERIA_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          selectOptions: { type: :array, items: { type: :string } }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Criterion'
      it_behaves_like 'Unprocessable Entity',
                      as: 'Criterion',
                      with_params: { select_options: [] },
                      on: :update

      response 200, 'Criterion updated' do
        let(:id) { criterion.id }
        let(:params) { { select_options: criterion.select_options.append('new option') } }

        schema '$ref' => 'swagger/schemas/admin/criteria/show.json'

        it 'updates a criterion' do |example|
          submit_request(example.metadata)
          criterion = Criterion.find(json_response[:id])
          expect(criterion.select_options).to eq params[:select_options]
        end
      end
    end

    delete 'Archives a Criterion' do
      include_context 'RequestDefaultParams', ADMIN_CRITERIA_TAG, true

      it_behaves_like 'Not Found', resourse_name: 'Criterion'
      let!(:id) { create(:criterion).id }

      response 204, 'Criterion archived' do
        it 'archives the criterion' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Criterion.archived, :count).by(1))

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/criteria/{id}/update_position' do
    parameter name: :id, in: :path, type: :integer

    put 'Updates criterions position' do
      include_context 'RequestDefaultParams', ADMIN_CRITERIA_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[position],
        properties: {
          position: { type: :integer, minimum: 1 }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Criterion'

      response 204, 'Criterions position updated' do
        let!(:positional_criterion) { create(:criterion, :mfc) }
        let!(:positional_criterion2) { create(:criterion, :mfc) }
        let(:id) { positional_criterion2.id }
        let(:params) { { position: Criterion.mfc.maximum(:position) - 1 } }

        it 'updates a criterion position' do |example|
          before = Criterion.find(id).position
          submit_request(example.metadata)
          after = Criterion.find(id).position
          expect(after).to eq(before - 1)

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end
end
