# frozen_string_literal: true

require 'swagger_helper'

describe 'Admin Inspections API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_INSPECTIONS_TAG = 'Admin Inspections'

  sign_in_superadmin

  let_it_be(:inspection) { create(:inspection, ends_at: 1.year.from_now) }

  before { Inspection.reindex }

  path '/inspections' do
    get 'Gets inspections' do
      include_context 'RequestDefaultParams', ADMIN_INSPECTIONS_TAG
      include_context 'SearchDefaultParams', InspectionSearcher

      it_behaves_like 'API Paginatable', :inspection

      response 200, 'Inspections found' do
        schema '$ref' => 'swagger/schemas/admin/inspections/index.json'
        run_test!
      end
    end

    post 'Creates an inspection' do
      include_context 'RequestDefaultParams', ADMIN_INSPECTIONS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[inspectableId inspectableType checklistId date startsAt],
        properties: {
          inspectableId: { type: :integer },
          inspectableType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values },
          checklistId: { type: :integer },
          date: { type: :string, format: :date },
          startsAt: { type: :string, format: :'date-time' },
          inspectorId: { type: :integer }
        }
      }

      it_behaves_like 'Unprocessable Entity',
                      as: 'Inspection',
                      with_params: { date: '' },
                      on: :create

      response 201, 'Inspection created' do
        let!(:calendar_day) { create(:calendar_day, date: date, user: inspector) }
        let(:date) { Date.current }
        let(:inspectable) { create(:mfc) }
        let(:checklist) { create(:checklist, project_type: 'mfc') }
        let(:inspector) { create(:user, project_type: 'mfc') }

        let(:params) do
          { date: date,
            starts_at: Time.current,
            ends_at: Time.current + 3.hours,
            inspectable_id: inspectable.id,
            inspectable_type: 'mfc',
            checklist_id: checklist.id,
            inspector_id: inspector.id }
        end

        schema '$ref' => 'swagger/schemas/admin/inspections/show.json'

        it 'creates an inspection' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Inspection, :count).by(1))

          inspection = Inspection.find(json_response[:id])

          params.keys.without(:starts_at, :ends_at, :inspectable_type).each do |attribute|
            expect(inspection.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/inspections/{id}' do
    parameter name: :id, in: :path, type: :integer, example: 1

    get 'Gets an inspection by ID' do
      include_context 'RequestDefaultParams', ADMIN_INSPECTIONS_TAG

      it_behaves_like 'Not Found', resource_name: 'Inspection'
      let(:id) { create(:inspection).id }

      response 200, 'Inspection found' do
        schema '$ref' => 'swagger/schemas/admin/inspections/show.json'
        run_test!
      end
    end

    put 'Updates an inspection' do
      include_context 'RequestDefaultParams', ADMIN_INSPECTIONS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          inspectableId: { type: :integer },
          inspectableType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values },
          checklistId: { type: :integer },
          date: { type: :string, format: :date },
          startsAt: { type: :string, format: :'date-time' },
          inspectorId: { type: :integer },
          rawData: { type: :string },
          state: { type: :string, enum: %w[started finished] }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Unprocessable Entity',
                      as: 'Inspection',
                      with_params: { date: '' },
                      on: :update

      response 200, 'Inspection updated' do
        let!(:calendar_day) { create(:calendar_day, date: date, user: inspector) }
        let(:date) { Date.current + 1.day }
        let(:id) { inspection.id }
        let(:inspectable) { create(:mfc) }
        let(:checklist) { create(:checklist, project_type: 'mfc') }
        let(:inspector) { create(:user, project_type: 'mfc') }

        let(:params) do
          { date: date,
            starts_at: Time.current + 1.1.days,
            inspectable_id: inspectable.id,
            inspectable_type: 'mfc',
            checklist_id: checklist.id,
            inspector_id: inspector.id }
        end

        schema '$ref' => 'swagger/schemas/admin/inspections/show.json'

        it 'updates an inspection' do |example|
          submit_request(example.metadata)

          inspection = Inspection.find(json_response[:id])
          params.keys.without(:starts_at, :inspectable_type).each do |attribute|
            expect(inspection.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/inspections/cancel' do
    put 'cancel inspections' do
      include_context 'RequestDefaultParams', ADMIN_INSPECTIONS_TAG, true

      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          cancelReason: { type: :string },
          inspectionIds: { type: :array, items: { type: :integer } }
        }
      }

      response 200, 'Inspections canceled' do
        let(:inspections) { create_list(:inspection, 2, state: :planned) }
        let(:params) do
          {
            cancel_reason: Inspection::CANCEL_REASONS[:sick],
            inspection_ids: inspections.map(&:id)
          }
        end

        schema '$ref' => 'swagger/schemas/admin/inspections/cancel.json'

        it 'cancels inspections' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Inspection.canceled, :count).by(2))

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/inspections/mass_destroy' do
    put 'destroy inspections' do
      include_context 'RequestDefaultParams', ADMIN_INSPECTIONS_TAG, true

      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          inspectionIds: { type: :array, items: { type: :integer } }
        }
      }

      response 204, 'Inspections destroyed' do
        let(:inspections) { create_list(:inspection, 2, state: :planned) }
        let(:params) do
          {
            inspection_ids: inspections.map(&:id)
          }
        end
        run_test!
      end
    end
  end
end
