# frozen_string_literal: true

require 'swagger_helper'

describe 'Admin MFC API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_MFCS_TAG = 'Admin MFC'

  sign_in_superadmin

  let_it_be(:mfc) { create(:mfc) }

  before { Mfc.reindex }

  path '/mfcs' do
    get 'Gets all MFC' do
      include_context 'RequestDefaultParams', ADMIN_MFCS_TAG, true
      include_context 'SearchDefaultParams', MfcSearcher

      it_behaves_like 'API Paginatable', :mfc

      response 200, 'MFCs found' do
        schema '$ref' => 'swagger/schemas/admin/mfcs/index.json'
        run_test!
      end
    end

    post 'Creates an MFC' do
      include_context 'RequestDefaultParams', ADMIN_MFCS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[name districtId territoryId address latitude longitude division kind],
        properties: {
          name: { type: :string },
          districtId: { type: :integer },
          territoryId: { type: :integer },
          address: { type: :string },
          latitude: { type: :string },
          longitude: { type: :string },
          division: { type: :string, enum: Mfc::DIVISIONS },
          kind: { type: :string, enum: Mfc::KINDS.values },
          comment: { type: :string }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Unprocessable Entity',
                      as: 'Mfc',
                      with_params: { name: '' },
                      on: :create

      response 201, 'MFC created' do
        let(:district) { create(:district) }
        let(:territory) { create(:territory) }

        let(:params) do
          attributes_for(:mfc).merge(district_id: district.id, territory_id: territory.id)
        end

        schema '$ref' => 'swagger/schemas/admin/mfcs/show.json'

        it 'persists valid attributes' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Mfc, :count).by(1))

          mfc = Mfc.find(json_response[:id])

          params.keys.each do |attribute|
            expect(mfc.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/mfcs/{id}' do
    parameter name: :id, in: :path, type: :integer, example: 1

    get 'Gets an MFC by id' do
      include_context 'RequestDefaultParams', ADMIN_MFCS_TAG

      it_behaves_like 'Not Found', resource_name: 'Mfc'

      let(:id) { create(:mfc).id }

      response 200, 'MFC found' do
        schema '$ref' => 'swagger/schemas/admin/mfcs/show.json'
        run_test!
      end
    end

    put 'Updates a MFC' do
      include_context 'RequestDefaultParams', ADMIN_MFCS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[name districtId territoryId address latitude longitude division kind],
        properties: {
          name: { type: :string },
          districtId: { type: :integer },
          territoryId: { type: :integer, description: 'Выпадаха на ТО1..20' },
          address: { type: :string },
          latitude: { type: :string },
          longitude: { type: :string },
          division: { type: :string, enum: Mfc::DIVISIONS },
          kind: { type: :string, enum: Mfc::KINDS.values },
          comment: { type: :string }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Mfc'
      it_behaves_like 'Unprocessable Entity',
                      as: 'Mfc',
                      with_params: { name: '' },
                      on: :update

      response 200, 'Mfc updated' do
        let(:id) { mfc.id }
        let(:another_district) { create(:district) }
        let(:another_territory) { create(:territory) }

        let(:params) do
          attributes_for(:mfc).merge(district_id: another_district.id, territory_id: another_territory.id)
        end

        schema '$ref' => 'swagger/schemas/admin/mfcs/show.json'

        it 'updates attributes' do |example|
          submit_request(example.metadata)

          mfc = Mfc.find(json_response[:id])

          params.keys.each do |attribute|
            expect(mfc.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end

    delete 'Arcvhies an MFC' do
      include_context 'RequestDefaultParams', ADMIN_MFCS_TAG, true

      it_behaves_like 'Not Found', resource_name: 'Mfc'

      let(:mfc) { create(:mfc) }
      let(:id) { mfc.id }

      response 204, 'Mfc archived' do
        it 'archives the mfc' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Mfc.archived, :count).by(1))
        end
      end

      response 422, 'Mfc cannot be archvied' do
        let!(:inspection) { create(:inspection, inspectable: mfc) }

        schema '$ref' => 'swagger/schemas/shared/failed_create.json'

        run_test!
      end
    end
  end
end
