# frozen_string_literal: true

require 'swagger_helper'

RSpec.describe 'Admin Policlinics API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_POLICLINICS_TAG = 'Admin Policlinics'

  sign_in_superadmin

  let_it_be(:policlinics) { create_list(:policlinic, 2) }
  let_it_be(:policlinic) { create(:policlinic) }

  before { Policlinic.reindex }

  path '/policlinics' do
    get 'Gets the administered policlinics' do
      include_context 'RequestDefaultParams', ADMIN_POLICLINICS_TAG
      include_context 'SearchDefaultParams', PoliclinicSearcher

      # it_behaves_like 'API Authenticable'
      it_behaves_like 'API Paginatable', :policlinic

      response '200', 'Policlinics found' do
        schema '$ref' => 'swagger/schemas/admin/policlinics/index.json'
        run_test!
      end
    end

    post 'Creates a policlinic' do
      include_context 'RequestDefaultParams', ADMIN_POLICLINICS_TAG, true
      parameter name: :params, in: :body, schema:     {
        type: :object,
        required: %i[name districtId address latitude longitude kind size],
        properties: {
          name: { type: :string },
          districtId: { type: :integer },
          address: { type: :string },
          latitude: { type: :string },
          longitude: { type: :string },
          population: { type: :integer },
          floors: { type: :integer },
          kind: { type: :string, enum: Policlinic::KINDS.values },
          size: { type: :string, enum: Policlinic::SIZES.values },
          comment: { type: :string },
          timeToInspectMinutes: { type: :integer }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Unprocessable Entity',
                      as: 'Policlinic',
                      with_params: { name: '' },
                      on: :create

      response 201, 'Policlinic created' do
        let(:district) { create(:district) }
        let(:params) do
          attributes_for(:policlinic).merge(district_id: district.id)
        end

        schema '$ref' => 'swagger/schemas/admin/policlinics/show.json'

        it 'persists valid attributes' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Policlinic, :count).by(1))

          policlinic = Policlinic.find(json_response[:id])

          params.keys.each do |attribute|
            expect(policlinic.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/policlinics/{id}' do
    parameter name: :id, in: :path, type: :integer, example: 1

    get 'Gets a policlinic by id' do
      include_context 'RequestDefaultParams', ADMIN_POLICLINICS_TAG

      # it_behaves_like 'API Authenticable'
      it_behaves_like 'Not Found', resource_name: 'Policlinic'

      let(:id) { create(:policlinic).id }

      response '200', 'Policlinic found' do
        schema '$ref' => 'swagger/schemas/admin/policlinics/show.json'
        run_test!
      end
    end

    put 'Updates a policlinic' do
      include_context 'RequestDefaultParams', ADMIN_POLICLINICS_TAG, true
      parameter name: :params, in: :body, schema:     {
        type: :object,
        required: %i[name districtId address latitude longitude kind size],
        properties: {
          name: { type: :string },
          districtId: { type: :integer },
          address: { type: :string },
          latitude: { type: :string },
          longitude: { type: :string },
          population: { type: :integer },
          floors: { type: :integer },
          kind: { type: :string, enum: Policlinic::KINDS.values },
          size: { type: :string, enum: Policlinic::SIZES.values },
          comment: { type: :string },
          timeToInspectMinutes: { type: :integer }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Policlinic'
      it_behaves_like 'Unprocessable Entity',
                      as: 'Policlinic',
                      with_params: { name: '' },
                      on: :update

      response 200, 'Policlinic updated' do
        let(:id) { policlinic.id }
        let(:another_district) { create(:district) }

        let(:params) do
          attributes_for(:policlinic)
            .merge(district_id: another_district.id)
        end

        schema '$ref' => 'swagger/schemas/admin/policlinics/show.json'

        it 'updates attributes' do |example|
          submit_request(example.metadata)

          policlinic = Policlinic.find(json_response[:id])

          params.keys.each do |attribute|
            expect(policlinic.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end

    delete 'Archives a policlinic' do
      include_context 'RequestDefaultParams', ADMIN_POLICLINICS_TAG, true

      it_behaves_like 'Not Found', resource_name: 'Policlinic'

      let(:policlinic) { create(:policlinic) }
      let(:id) { policlinic.id }

      response 204, 'Policlinic archvied' do
        it 'archives the policlinic' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Policlinic.archived, :count).by(1))
        end
      end

      response 422, 'Policlinic cannot be archived' do
        let!(:inspection) { create(:inspection, inspectable: policlinic) }

        schema '$ref' => 'swagger/schemas/shared/failed_create.json'

        run_test!
      end
    end
  end
end
