# frozen_string_literal: true

require 'swagger_helper'

describe 'Admin Repair Periods API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_REPAIR_PERIOS_TAG = 'Admin Repair Periods'

  sign_in_superadmin

  path '/repair_periods' do
    post 'Creates a repair period' do
      include_context 'RequestDefaultParams', ADMIN_REPAIR_PERIOS_TAG, true
      parameter name: :params, in: :body, schema: {
        required: %i[startsAt repairableId repairableType],
        properties: {
          startsAt: { type: :string, format: :'date-time' },
          endsAt: { type: :string, format: :'date-time' },
          repairableId: { type: :integer },
          repairableType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values }
        }
      }

      it_behaves_like 'Unprocessable Entity',
                      as: 'RepairPeriod',
                      with_params: { startsAt: '' },
                      on: :create

      response 201, 'Repair Period created' do
        let(:repairable) { create(:mfc) }
        let(:params) { attributes_for(:repair_period).merge(repairable_id: repairable.id, repairable_type: 'mfc') }

        schema '$ref' => 'swagger/schemas/admin/repair_periods/_repair_period.json'

        it 'creates a repair period' do |example|
          expect { submit_request(example.metadata) }
            .to(change(repairable.repair_periods, :count).by(1))

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/repair_periods/{id}' do
    parameter name: :id, in: :path, type: :integer, example: 1

    put 'Updates a repair period' do
      include_context 'RequestDefaultParams', ADMIN_REPAIR_PERIOS_TAG, true

      parameter name: :params, in: :body, schema: {
        properties: {
          startsAt: { type: :string, format: :'date-time' },
          endsAt: { type: :string, format: :'date-time' },
          repairableId: { type: :integer },
          repairableType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'RepairPeriod'
      it_behaves_like 'Unprocessable Entity',
                      as: 'RepairPeriod',
                      with_params: { startsAt: Time.current, ends_at: Time.current - 1.day },
                      on: :update

      response 200, 'RepairPeriod updated' do
        let(:repair_period) { create(:repair_period) }
        let!(:id) { repair_period.id }

        let(:params) { { ends_at: repair_period.ends_at + 5.days } }

        schema '$ref' => 'swagger/schemas/admin/repair_periods/_repair_period.json'

        it 'updates a repair period' do |example|
          submit_request(example.metadata)

          repair_period = RepairPeriod.find(json_response[:id])
          expect(repair_period.ends_at).to be_within(1.second).of(params[:ends_at])
          assert_response_matches_metadata(example.metadata)
        end
      end
    end

    delete 'Deletes a repair period' do
      include_context 'RequestDefaultParams', ADMIN_REPAIR_PERIOS_TAG, true

      it_behaves_like 'Not Found', resource_name: 'Repair Period'

      response 204, 'Repair Period deleted' do
        let!(:id) { create(:repair_period).id }

        it 'deletes a repair period' do |example|
          expect { submit_request(example.metadata) }
            .to change(RepairPeriod, :count).by(-1)

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end
end
