# frozen_string_literal: true

require 'swagger_helper'

describe 'Admin Users API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_USERS_TAG = 'Admin Users'

  sign_in_superadmin

  let_it_be(:users) { create_list(:user, 2) }
  let_it_be(:user) { create(:user, :policlinic, :with_planned_inspections) }
  let_it_be(:curator) { create(:user, :mfc, role: User::ROLES[:curator]) }

  before { User.reindex }

  path '/users' do
    get 'Gets administered users' do
      include_context 'RequestDefaultParams', ADMIN_USERS_TAG, true
      include_context 'SearchDefaultParams', UserSearcher

      it_behaves_like 'API Paginatable', :user

      response '200', 'Users found' do
        schema '$ref' => 'swagger/schemas/admin/users/index.json'
        run_test!
      end
    end

    post 'Creates a user' do
      include_context 'RequestDefaultParams', ADMIN_USERS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[firstName lastName email role projectType],
        properties: {
          firstName: { type: :string },
          lastName: { type: :string },
          middleName: { type: :string },
          email: { type: :string },
          phone: { type: :string },
          role: { type: :string, enum: User::ROLES.values },
          projectType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values },
          gender: { type: :string, enum: User::GENDERS.values },
          curatorId: { type: :integer }
        }
      }

      let(:params) { {} }

      # it_behaves_like 'API Authenticable'
      it_behaves_like 'Unprocessable Entity',
                      as: 'User',
                      with_params: { first_name: '' },
                      on: :create

      response 201, 'User created' do
        let(:params) do
          attributes_for(:user).slice(:first_name, :last_name, :middle_name, :email, :phone, :role, :project_type, :gender)
        end

        schema '$ref' => 'swagger/schemas/admin/users/show.json'

        it 'creates a user' do |example|
          expect(Auth::PasswordsMailer).to receive(:invite).and_call_original
          expect { submit_request(example.metadata) }
            .to(change(User, :count).by(1))

          user = User.find(json_response[:id])
          # ensure all params we've sent persisted as attributes of created user
          params.keys.each do |attribute|
            expect(user.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/users/{id}' do
    parameter name: :id, in: :path, type: :integer, default: 1

    get 'Gets a user by ID' do
      include_context 'RequestDefaultParams', ADMIN_USERS_TAG

      it_behaves_like 'Not Found', resource_name: 'User'
      let(:id) { create(:user).id }

      response 200, 'User found' do
        schema '$ref' => 'swagger/schemas/admin/users/show.json'
        run_test!
      end
    end

    put 'Updates a user' do
      include_context 'RequestDefaultParams', ADMIN_USERS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[firstName lastName email role projectType],
        properties: {
          firstName: { type: :string },
          lastName: { type: :string },
          middleName: { type: :string },
          email: { type: :string },
          phone: { type: :string },
          role: { type: :string, enum: User::ROLES.values },
          projectType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values },
          gender: { type: :string, enum: User::GENDERS.values },
          curatorId: { type: :integer }
        }
      }

      let(:params) { {} }

      # it_behaves_like 'API Authenticable'
      it_behaves_like 'Not Found', resource_name: 'User'
      it_behaves_like 'Unprocessable Entity',
                      as: 'User',
                      with_params: { first_name: '' },
                      on: :update

      response '200', 'User updated' do
        let(:id) { user.id }
        let(:params) do
          attributes_for(:user).slice(:first_name, :last_name, :middle_name, :email, :phone, :role, :project_type, :gender)
        end

        schema '$ref' => 'swagger/schemas/admin/users/show.json'

        it 'updates a user' do |example|
          submit_request(example.metadata)

          user = User.find(json_response[:id])
          params.keys.each do |attribute|
            expect(user.send(attribute)).to eq params[attribute]
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end

    delete 'Archives a user' do
      include_context 'RequestDefaultParams', ADMIN_USERS_TAG, true

      it_behaves_like 'Not Found', resource_name: 'User'

      sign_in_superadmin

      let(:user) { create(:user) }
      let(:id) { user.id }

      response 204, 'User archived' do
        it 'archives the user' do |example|
          expect { submit_request(example.metadata) }
            .to(change(User.archived, :count).by(1))

          assert_response_matches_metadata(example.metadata)
        end
      end

      response 422, 'User not archived' do
        let(:user) { current_user }
        let(:id) { user.id }

        schema '$ref' => 'swagger/schemas/shared/failed_create.json'

        run_test!
      end
    end
  end

  path '/users/{id}/reactivate' do
    parameter name: :id, in: :path, type: :integer

    put 'Reactivates archived user' do
      include_context 'RequestDefaultParams', ADMIN_USERS_TAG, true

      it_behaves_like 'Not Found', resource_name: 'User'

      response 204, 'User reactivated' do
        let!(:user) { create(:user, :archived) }
        let(:id) { user.id }

        it 'reactivates user' do |example|
          expect { submit_request(example.metadata) }
            .to(change(User.archived, :count).by(-1))

          assert_response_matches_metadata(example.metadata)
        end
      end

      response 422, 'User cannot be reactivated' do
        let(:user) { create(:user) }
        let(:id) { user.id }

        it 'does not reactivate user' do |example|
          expect { submit_request(example.metadata) }
            .not_to(change(User.registered, :count))

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/users/mass_update' do
    parameter name: :params, in: :body, schema: {
      type: :object,
      properties: {
        projectType: { type: :string, enum: ProjectTypes::ALLOWED_TYPES.values },
        curatorId: { type: :integer },
        cancelInspections: { type: :boolean },
        destroyInspections: { type: :boolean },
        purgeStartsAt: { type: :string }
      }
    }

    let(:params) { {} }

    put 'Updates target users project type and curator' do
      include_context 'RequestDefaultParams', ADMIN_USERS_TAG, true

      response '200', 'Users updated' do
        let(:params) { { project_type: 'mfc', curator_id: curator.id, user_ids: users.map(&:id) } }

        schema '$ref' => 'swagger/schemas/admin/users/mass_update.json'

        it 'updates a user project type and curator' do |example|
          submit_request(example.metadata)

          user_ids = json_response[:users].map { |el| el[:id] }
          users = User.where(id: user_ids)
          users.each do |user|
            expect(user.project_type).to eq(params[:project_type])
            expect(user.curator_id).to eq(params[:curator_id])
          end

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end
end
