# frozen_string_literal: true

require 'swagger_helper'

describe 'Admin Vacations API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_VACATIONS_TAG = 'Admin Vacations'

  sign_in_superadmin
  let_it_be(:vacation) { create(:vacation) }

  before { Vacation.reindex }

  path '/vacations' do
    get 'Gets administered users vacations' do
      include_context 'RequestDefaultParams', ADMIN_VACATIONS_TAG
      include_context 'SearchDefaultParams', VacationSearcher

      it_behaves_like 'API Paginatable', :vacation

      response 200, 'Vacations found' do
        schema '$ref' => 'swagger/schemas/admin/vacations/index.json'
        run_test!
      end
    end

    post 'Creates a vacation' do
      include_context 'RequestDefaultParams', ADMIN_VACATIONS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[userId startsAt endsAt],
        properties: {
          userId: { type: :integer },
          startsAt: { type: :string, format: :date },
          endsAt: { type: :string, format: :date }
        }
      }

      # TODO: write manually
      # it_behaves_like 'Unprocessable Entity',
      #   as: 'Vacation',
      #   with_params: { user_id: create(:user).id },
      #   on: :create

      let(:params) { {} }
      it_behaves_like 'Not Found', resource_name: 'User'

      response 201, 'Vacation created' do
        let(:user) { create(:user, :with_curator) }
        let(:params) { attributes_for(:vacation).merge(user_id: user.id) }

        schema '$ref' => 'swagger/schemas/admin/vacations/show.json'

        it 'creates a vacation' do |example|
          expect { submit_request(example.metadata) }
            .to(change(Vacation, :count).by(1))

          vacation = Vacation.find(json_response[:id])

          expect(vacation.calendar_days).not_to be_empty
          expect(vacation.user).to eq user
        end
      end
    end
  end

  path '/vacations/{id}' do
    parameter name: :id, in: :path, type: :integer, example: 1

    get 'Gets a vacation by ID' do
      include_context 'RequestDefaultParams', ADMIN_VACATIONS_TAG

      it_behaves_like 'Not Found', resource_name: 'Vacation'
      let(:id) { create(:vacation).id }

      response 200, 'Vacation found' do
        schema '$ref' => 'swagger/schemas/admin/vacations/show.json'
        run_test!
      end
    end

    put 'Updates a vacation' do
      include_context 'RequestDefaultParams', ADMIN_VACATIONS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          userId: { type: :integer },
          startsAt: { type: :string, format: :date },
          endsAt: { type: :string, format: :date }
        }
      }

      let(:params) { {} }

      # TODO: write manually
      # it_behaves_like 'Unprocessable Entity'

      response 200, 'Vacation updated' do
        let(:vacation) { create(:vacation) }
        let(:user_id) { create(:user, :with_curator).id }
        let(:id) { vacation.id }

        let(:params) { { starts_at: vacation.starts_at + 10.days, ends_at: vacation.ends_at + 10.days, user_id: user_id } }

        schema '$ref' => 'swagger/schemas/admin/vacations/show.json'

        it 'updates a vacation' do |example|
          submit_request(example.metadata)
          expect(vacation.reload.starts_at).to eq params[:starts_at]
          expect(vacation.ends_at).to eq params[:ends_at]
          expect(vacation.user_id).to eq user_id

          # expect vacation's calendar days to be updated
          expect(vacation.calendar_days.map(&:date))
            .to eq((params[:starts_at]..params[:ends_at]).to_a)

          assert_response_matches_metadata(example.metadata)
        end
      end
    end

    delete 'Deletes a vacation' do
      include_context 'RequestDefaultParams', ADMIN_VACATIONS_TAG, true

      it_behaves_like 'Not Found', resource_name: 'Vacation'

      let!(:vacation) { create(:vacation) }
      let(:id) { vacation.id }

      response 204, 'Vacation deleted' do
        it 'destroys the vacation' do |example|
          expect { submit_request(example.metadata) }
            .to change(Vacation, :count).by(-1)
                                        .and(change(CalendarDay.vacation, :count).by(-3))

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end
end
