# frozen_string_literal: true

require 'swagger_helper'

describe 'Admin Working Hours Norms API', type: :request, swagger_doc: 'admin-swagger.json' do
  ADMIN_WORKING_NORMS_TAG = 'Admin Working Norms'

  sign_in_superadmin

  let_it_be(:working_norms) { create_list(:working_norm, 2) }
  let_it_be(:working_norm) { create(:working_norm, date: '2077-01-01') }

  path '/working_norms' do
    get 'Gets working norms' do
      include_context 'RequestDefaultParams', ADMIN_WORKING_NORMS_TAG

      parameter name: :page, in: :query, type: :integer
      parameter name: :perPage, in: :query, type: :integer, default: Pagy::VARS[:items]

      let(:page) { 1 }
      let(:perPage) { 99 }

      it_behaves_like 'API Paginatable', :working_norm

      response 200, 'Working norms found' do
        schema '$ref' => 'swagger/schemas/admin/working_norms/index.json'
        run_test!
      end
    end

    post 'Creates a working norm' do
      include_context 'RequestDefaultParams', ADMIN_WORKING_NORMS_TAG, true
      parameter name: :params, in: :body, schema: {
        type: :object,
        required: %i[date workingHoursNorms],
        properties: {
          date: { type: :string, format: :date },
          workingHoursNorms: { type: :integer }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Unprocessable Entity',
                      as: 'WorkingNorm',
                      with_params: { date: '' },
                      on: :create

      response 201, 'Working Norm created' do
        let(:params) { attributes_for(:working_norm, working_hours_norms: 170, date: '1994-01-01') }

        schema '$ref' => 'swagger/schemas/admin/working_norms/show.json'

        it 'creates a calendar month' do |example|
          expect { submit_request(example.metadata) }
            .to(change(WorkingNorm, :count).by(1))

          result = WorkingNorm.find(json_response[:id])

          expect(result.hours).to eq params[:working_hours_norms]

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/working_norms/{id}' do
    parameter name: :id, in: :path, type: :integer

    get 'Gets a Working Norm by ID' do
      include_context 'RequestDefaultParams', ADMIN_WORKING_NORMS_TAG

      it_behaves_like 'Not Found', resource_name: 'Working Norm'
      let(:id) { create(:working_norm, date: '1999-01-01').id }

      response 200, 'Working Norm found' do
        schema '$ref' => 'swagger/schemas/admin/working_norms/show.json'
        run_test!
      end
    end

    put 'Updates a Working Norm by ID' do
      include_context 'RequestDefaultParams', ADMIN_WORKING_NORMS_TAG, true
      parameter name: :params, in: :body, schema: {
        properties: {
          date: { type: :string, format: :date },
          workingHoursNorms: { type: :integer }
        }
      }

      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Working Norm'
      it_behaves_like 'Unprocessable Entity',
                      as: 'WorkingNorm',
                      with_params: { date: '' },
                      on: :update

      response 200, 'Working Norm updated' do
        let(:id) { working_norm.id }
        let(:params) { { working_hours_norms: 3000 } }

        schema '$ref' => 'swagger/schemas/admin/working_norms/show.json'

        it 'updates a Working Norm' do |example|
          submit_request(example.metadata)

          working_norm = WorkingNorm.find(json_response[:id])

          expect(working_norm.hours).to eq params[:working_hours_norms]

          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end
end
