# frozen_string_literal: true

require 'swagger_helper'

RSpec.describe 'Auth Passwords API', type: :request, swagger_doc: 'auth-swagger.json' do
  AUTH_PASSWORDS_TAG = 'Passwords'

  path '/passwords/forgot', type: :request do
    post 'Sends email with recovery password link' do
      include_context 'RequestDefaultParams', AUTH_PASSWORDS_TAG
      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          email: { type: :string, example: 'vasya@pupkin.test' }
        },
        required: %w[email]
      }

      response '204', 'Recovery password link sent' do
        let!(:user) { create(:user, state: :registered) }
        let(:params) { { email: user.email.upcase } }

        it 'returns a 204 response' do |example|
          expect(Auth::PasswordsMailer).to receive(:forgot).and_call_original
          submit_request(example.metadata)
        end
      end

      response '404', 'User not found' do
        let!(:user) { create(:user, :archived) }
        let(:params) { { email: user.email.upcase } }

        schema '$ref' => 'swagger/schemas/shared/error.json'
        run_test!
      end
    end
  end

  path '/passwords/check_token_expiration', type: :request do
    post 'Checks token expiration' do
      include_context 'RequestDefaultParams', AUTH_PASSWORDS_TAG
      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          token: { type: :string, example: SecureRandom.hex(32) }
        },
        required: %w[token]
      }

      response '204', 'Token not expired' do
        let!(:user) { create(:user, recovery_password_token_sent_at: Time.current, recovery_password_token: 'token' * 7) }
        let(:params) { { token: user.recovery_password_token } }

        run_test!
      end

      response '404', 'Token not found' do
        let!(:user) { create(:user, recovery_password_token_sent_at: Time.current, recovery_password_token: 'token' * 7) }
        let(:params) { { token: 'invalid token' } }

        schema '$ref' => 'swagger/schemas/shared/error.json'
        run_test!
      end

      response '422', 'Token expired' do
        let!(:user) do
          create(:user, recovery_password_token_sent_at: User::RECOVERY_PASSWORD_TOKEN_EXP_TIME.ago,
                        recovery_password_token: 'token' * 7)
        end
        let(:params) { { token: user.recovery_password_token } }

        schema '$ref' => 'swagger/schemas/shared/error.json'
        run_test!
      end
    end
  end

  path '/passwords/reset', type: :request do
    post 'Resets password and automaticly signs in user' do
      include_context 'RequestDefaultParams', AUTH_PASSWORDS_TAG
      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          token: { type: :string, example: SecureRandom.hex(32) },
          password: { type: :string, example: 'new_password' }
        },
        required: %w[token password]
      }

      response '200', 'Password reset' do
        let!(:user) { create(:user, recovery_password_token_sent_at: Time.current, recovery_password_token: 'token' * 7) }
        let(:params) { { token: user.recovery_password_token, password: 'new_password' } }

        schema '$ref' => 'swagger/schemas/auth/sessions/create.json'
        run_test! do
          expect(user.reload.recovery_password_token_sent_at).to eq(nil)
          expect(user.recovery_password_token).to eq(nil)
          expect(user.state).to eq('registered')
        end
      end

      response '404', 'Token not found' do
        let!(:user) { create(:user, recovery_password_token_sent_at: Time.current, recovery_password_token: 'token' * 7) }
        let(:params) { { token: 'invalid token', password: 'new_password' } }

        schema '$ref' => 'swagger/schemas/shared/error.json'
        run_test!
      end

      response '422', 'Token expired' do
        let!(:user) do
          create(:user, recovery_password_token_sent_at: User::RECOVERY_PASSWORD_TOKEN_EXP_TIME.ago,
                        recovery_password_token: 'token' * 7)
        end
        let(:params) { { token: user.recovery_password_token, password: 'new_password' } }

        schema '$ref' => 'swagger/schemas/shared/error.json'
        run_test!
      end
    end
  end

  path '/passwords/send_new_link', type: :request do
    post 'Sends new link' do
      include_context 'RequestDefaultParams', AUTH_PASSWORDS_TAG
      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          token: { type: :string, example: SecureRandom.hex(32) }
        },
        required: %w[token]
      }

      response '204', 'New link sent' do
        let!(:user) do
          create(:user, recovery_password_token_sent_at: User::RECOVERY_PASSWORD_TOKEN_EXP_TIME.ago,
                        recovery_password_token: 'token' * 7)
        end
        let(:params) { { token: user.recovery_password_token, password: 'new_password' } }

        it 'returns a 204 response' do |example|
          expect(Auth::PasswordsMailer).to receive(:invite).and_call_original
          submit_request(example.metadata)
        end
      end

      response '404', 'Token not found' do
        let!(:user) { create(:user, recovery_password_token_sent_at: Time.current, recovery_password_token: 'token' * 7) }
        let(:params) { { token: 'invalid token', password: 'new_password' } }

        schema '$ref' => 'swagger/schemas/shared/error.json'
        run_test!
      end

      response '422', 'Token not expired yet' do
        let!(:user) { create(:user, recovery_password_token_sent_at: Time.current, recovery_password_token: 'token' * 7) }
        let(:params) { { token: user.recovery_password_token, password: 'new_password' } }

        schema '$ref' => 'swagger/schemas/shared/error.json'
        run_test!
      end
    end
  end
end
