# frozen_string_literal: true

require 'swagger_helper'

RSpec.describe 'Auth Sessions API', type: :request, swagger_doc: 'auth-swagger.json' do
  AUTH_SESSIONS_TAG = 'Sessions'

  path '/sessions/sign_in', type: :request do
    post 'Signs in and returns access cookie and csrf token, and employee object' do
      include_context 'RequestDefaultParams', AUTH_SESSIONS_TAG

      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          email: { type: :string, example: 'aanikin@royalsw.me' },
          password: { type: :string, example: 'password' }
        },
        required: %w[email password]
      }

      response '200', 'Successfully authenticated' do
        let!(:user) { create(:user) }
        let(:params) { { email: user.email.upcase, password: user.password } }

        schema '$ref' => 'swagger/schemas/auth/sessions/create.json'

        run_test! do
          expect(user.reload.last_sign_in_at).not_to be_nil
        end
      end

      response '401', 'Not authenticated' do
        schema '$ref' => 'swagger/schemas/shared/error.json'

        context 'with invalid credentials' do
          let(:params) { { email: 'vasya@pupkin.com', password: 'incorrect' } }

          it 'does not authenticate the user' do |example|
            submit_request(example.metadata)

            expect(response.cookies[JWTSessions.access_cookie]).not_to be_present
            assert_response_matches_metadata(example.metadata)
          end
        end

        context 'when user is archived' do
          let(:user) { create(:user, :archived) }
          let(:params) { { email: user.email, password: user.password } }

          it 'does not authenticate the user' do |example|
            expect { submit_request(example.metadata) }
              .not_to change(user, :last_sign_in_at)

            expect(response.cookies[JWTSessions.access_cookie]).not_to be_present
            assert_response_matches_metadata(example.metadata)
          end
        end
      end
    end
  end

  path '/sessions/refresh' do
    post 'Refreshes access cookie and csrf token' do
      include_context 'RequestDefaultParams', AUTH_SESSIONS_TAG, true

      response '200', 'Successfully refreshed when access cookie is expired' do
        before do
          # set expiration time to 0 to create an already expired access token
          user = create(:user)
          JWTSessions.access_exp_time = 0
          payload = { user_id: user.id }
          session = JWTSessions::Session.new(payload: payload, refresh_by_access_allowed: true)
          @tokens = session.login
          JWTSessions.access_exp_time = 3600
          cookies[JWTSessions.access_cookie] = @tokens[:access]
        end

        run_test! do
          expect(JSON.parse(response.body).keys.sort).to eq ['csrf']
          expect(response.cookies[JWTSessions.access_cookie]).to be_present
        end
      end

      response '401', "Not refreshed when acceess cookie doesn't expired" do
        sign_in_user
        schema '$ref' => 'swagger/schemas/shared/error.json'
        run_test!
      end
    end
  end

  path '/sessions/sign_out' do
    delete 'Signs out and deletes access cookie' do
      include_context 'RequestDefaultParams', AUTH_SESSIONS_TAG, true

      it_behaves_like 'API Authenticable', is_authorization_required: false

      sign_in_user

      response '200', 'Access cookie deleted' do
        run_test!
      end
    end
  end
end
