# frozen_string_literal: true

require 'swagger_helper'

# rubocop:disable RSpec/ScatteredSetup, RSpec/ExpectInHook, RSpec/AnyInstance
describe 'Inspections API', type: :request do
  INSPECTIONS_TAG = 'Inspections'

  let_it_be(:user) { create(:user, :inspector, :policlinic) }
  let_it_be(:inspections) { create_list(:inspection, 2, inspector: user, date: Time.zone.today) }
  let_it_be(:outdated_inspection) { create(:inspection, inspector: user, date: 1.day.ago) }

  before { sign_in user }

  path '/inspections' do
    get 'Gets inpections for specified date' do
      include_context 'RequestDefaultParams', INSPECTIONS_TAG, true

      parameter name: :params, in: :body, schema: {
        required: %i[date],
        properties: {
          date: { type: :string, format: :date }
        }
      }

      let(:params) { { date: Time.zone.today } }

      response 200, 'Inspections found' do
        schema '$ref' => 'swagger/schemas/inspections/index.json'
        run_test! do
          expect(json_response[:inspections].map { |e| e[:id] }).not_to include outdated_inspection.id
        end
      end
    end
  end

  path '/inspections/{id}' do
    parameter name: :id, in: :path, type: :integer

    get 'Gets an inspection by ID' do
      include_context 'RequestDefaultParams', INSPECTIONS_TAG

      it_behaves_like 'Not Found', resource_name: 'Inspection'
      let(:id) { create(:inspection, inspector: current_user).id }

      response 200, 'Inspection found' do
        schema '$ref' => 'swagger/schemas/inspections/show.json'
        run_test!
      end
    end

    put 'Updates an inspection by ID' do
      include_context 'RequestDefaultParams', INSPECTIONS_TAG, true
      parameter name: :params, in: :body, schema: {
        properties: {
          rawData: { type: :string },
          state: { type: :string, enum: %w[started finished] }
        }
      }

      let(:id) { inspection.id }
      let(:inspection) { create(:inspection, inspector: current_user) }
      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Inspection'
      it_behaves_like 'Unprocessable Entity',
                      as: 'Inspection',
                      with_params: { raw_data: '' },
                      on: :update

      response 200, 'Inspection updated' do
        let(:params) { { raw_data: %({ "blocks": [] }), state: 'started' } }

        schema '$ref' => 'swagger/schemas/inspections/show.json'
        run_test! do
          expect(inspection.reload.raw_data).to eq params[:raw_data]
          expect(inspection.state).to eq params[:state]
        end
      end
    end
  end

  path '/inspections/{id}/attach_file' do
    parameter name: :id, in: :path, type: :integer

    post 'Attaches a file to an inspection by signed id' do
      include_context 'RequestDefaultParams', INSPECTIONS_TAG, true
      parameter name: :params, in: :body, schema: {
        required: %i[signedId],
        properties: {
          signedId: { type: :string }
        }
      }

      let(:inspection) { create(:inspection, inspector: current_user) }
      let(:id) { inspection.id }
      let(:params) { { signed_id: '123' } }

      before do
        # stub out the need for having a blob with signed_id by pretending the @inspection.files.attach worked
        expect_any_instance_of(Inspection).to receive_message_chain('files.attach').and_return(true)
      end

      response 200, 'File attached to an inspection' do
        # TODO: figure this out or remove
        # Doesn't work, when calling ActiveStorage::Blob it raises
        # let(:blob_args) { {filename: 'filename.jpg', byte_size: 1, checksum: 1, content_type: 'image/jpg', metadata: {}} }
        # let(:blob) {   ActiveStorage::Blob.create_before_direct_upload!(blob_args) }
        # let(:params) { { signed_id: blob.signed_id }}

        run_test!
      end
    end
  end

  path '/inspections/{id}/purge_file' do
    parameter name: :id, in: :path, type: :integer

    delete 'Detaches and purges a file from an inspection by key' do
      include_context 'RequestDefaultParams', INSPECTIONS_TAG, true
      parameter name: :params, in: :body, schema: {
        required: %i[key],
        properties: {
          key: { type: :string }
        }
      }

      let(:inspection) { create(:inspection, inspector: current_user) }
      let(:id) { inspection.id }
      let(:params) { { key: 'some_key' } }

      before do
        expect_any_instance_of(Inspection).to receive_message_chain('files.joins.find_by!').and_return(ActiveStorage::Attachment.new)
        expect_any_instance_of(ActiveStorage::Attachment).to receive('purge').and_return(true)
      end

      response 204, 'File detached and purged' do
        run_test!
      end
    end
  end

  path '/inspections/{id}/start' do
    post 'Starts an inspection' do
      include_context 'RequestDefaultParams', INSPECTIONS_TAG, true

      let(:inspection) { create(:inspection, state: :planned, inspector: current_user) }
      let(:id) { inspection.id }
      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Inspection'

      parameter name: :id, in: :path, type: :integer

      response '204', 'Inspection started' do
        it 'returns a 204 response' do |example|
          expect { submit_request(example.metadata) }.to change { inspection.reload.state }.from('planned').to('started')
          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end

  path '/inspections/{id}/finish' do
    post 'Finishes an inspection' do
      include_context 'RequestDefaultParams', INSPECTIONS_TAG, true

      let(:inspection) { create(:inspection, state: :started, inspector: current_user) }
      let(:id) { inspection.id }
      let(:params) { {} }

      it_behaves_like 'Not Found', resource_name: 'Inspection'

      parameter name: :id, in: :path, type: :integer

      response '204', 'Inspection finished' do
        it 'returns a 204 response' do |example|
          expect { submit_request(example.metadata) }.to change { inspection.reload.state }.from('started').to('finished')
          assert_response_matches_metadata(example.metadata)
        end
      end
    end
  end
end
# rubocop:enable RSpec/ScatteredSetup, RSpec/ExpectInHook, RSpec/AnyInstance
