# frozen_string_literal: true

require 'rails_helper'

describe FileArchiveSearcher do
  let_it_be(:current_project_type) { 'mfc' }
  let_it_be(:parameter) { create(:parameter) }
  let_it_be(:searched) { create(:inspection, state: :finished, inspectable: create(:mfc)) }
  let_it_be(:searched_with_violation) { create(:inspection, :with_violated_parameter, state: :finished, inspectable: create(:mfc)) }
  let_it_be(:unrelated) { create(:inspection, state: :finished, inspectable: create(:mfc)) }
  let_it_be(:outdated) { create(:inspection, date: 1.month.ago, state: :finished, inspectable: create(:mfc)) }
  let_it_be(:base_filtered) { create(:inspection, state: :started, inspectable: create(:mfc)) }
  let_it_be(:user) { create(:user, :admin, project_type: current_project_type) }

  before_all { Inspection.reindex }

  subject(:searcher) { described_class.call(params, user, current_project_type) }

  describe 'filtering' do
    context 'by inspector_id' do
      let(:params) { { filters: [{ field: 'inspector_id', value: [searched.inspector_id] }] } }

      it 'works' do
        expect(searcher.results).to include searched
        expect(searcher.results).not_to include unrelated
      end
    end

    context 'by inspectable_id' do
      let(:params) { { filters: [{ field: 'inspectable_id', value: [searched.inspectable_id] }] } }

      it 'works' do
        expect(searcher.results).to include searched
        expect(searcher.results).not_to include unrelated
      end
    end

    context 'by date' do
      let(:params) { { filters: [{ field: 'date', range: { gte: searched.date - 1.day, lte: searched.date + 1.day } }] } }

      it 'works' do
        expect(searcher.results).to include searched
        expect(searcher.results).not_to include outdated
      end
    end

    context 'by parameter_id' do
      # подставной параметр с id 0 который дает трейт :with_violated_parameter
      let(:params) { { filters: [{ field: 'parameter_id', value: [0] }] } }

      it 'works' do
        expect(searcher.results).to include searched_with_violation
        expect(searcher.results).not_to include unrelated
      end
    end

    context 'by checklist_id' do
      let(:params) { { filters: [{ field: 'checklist_id', value: [searched.checklist_id] }] } }

      it 'works' do
        expect(searcher.results).to include searched
        expect(searcher.results).not_to include unrelated
      end
    end

    context 'after_filter' do
      let(:params) { {} }

      it 'works' do
        expect(searcher.results).not_to include(base_filtered)
      end
    end
  end
end
