# frozen_string_literal: true

require 'rails_helper'

describe InspectionSearcher do
  let_it_be(:current_project_type) { 'mfc' }
  let_it_be(:searched) { create(:inspection, date: 1.day.from_now, inspectable: create(:mfc)) }
  let_it_be(:sorted) { create(:inspection, date: 2.days.ago, inspectable: create(:mfc), state: :finished) }
  let_it_be(:unrelated) { create(:inspection, date: 5.days.from_now, inspectable: create(:policlinic), state: :finished) }
  let_it_be(:admin) { create(:user, :admin, project_type: 'mfc') }
  let_it_be(:curator) { create(:user, :curator, inspectors: [searched.inspector]) }
  let_it_be(:user) { create(:user, :superadmin) }

  before_all { Inspection.reindex }

  subject(:searcher) { described_class.call(params, user, current_project_type) }

  let(:params) { {} }

  describe 'filtering' do
    context 'by inspector_id' do
      let(:params) { { filters: [{ field: 'inspector_id', value: [searched.inspector_id] }] } }

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq searched
      end
    end

    context 'by inspectable_id' do
      let(:params) { { filters: [{ field: 'inspectable_id', value: [searched.inspectable_id] }] } }

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq searched
      end
    end

    context 'by checklist_id' do
      let(:params) { { filters: [{ field: 'checklist_id', value: [searched.checklist_id] }] } }

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq searched
      end
    end

    context 'by state' do
      let(:params) { { filters: [{ field: 'state', value: [searched.state] }] } }

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results).to match_array [searched]
      end
    end

    context 'by date' do
      let(:params) { { filters: [{ field: 'date', range: { gte: searched.date - 1.day, lte: searched.date + 1.day } }] } }

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq searched
      end
    end
  end

  describe 'after filter' do
    context 'as an admin of project' do
      let(:user) { admin }

      it 'does not show unrelated project inspections' do
        expect(searcher).not_to include(unrelated)
      end
    end

    context 'as curator' do
      let(:user) { curator }

      it 'only shows curated inspections' do
        expect(searcher.size).to eq 1
        expect(searcher.results).to include(searched)
      end
    end
  end

  describe 'sorting' do
    it_behaves_like 'Sortable', fields: %i[id date state], is_user_required: true, project_type_required: true

    context 'by inspectable_name' do
      context 'with ASC' do
        let(:params) { { sortings: [{ field: 'inspectable_name', value: 'asc' }] } }

        it 'sorts results' do
          expect(searcher.results.first.inspectable.name)
            .to be < searcher.results.last.inspectable.name
        end
      end

      context 'with DESC' do
        let(:params) { { sortings: [{ field: 'inspectable_name', value: 'desc' }] } }

        it 'sorts results' do
          expect(searcher.results.first.inspectable.name)
            .to be > searcher.results.last.inspectable.name
        end
      end
    end

    context 'by checklist_name' do
      context 'with ASC' do
        let(:params) { { sortings: [{ field: 'checklist_name', value: 'asc' }] } }

        it 'sorts results' do
          expect(searcher.results.first.checklist.name)
            .to be < searcher.results.last.checklist.name
        end
      end

      context 'with DESC' do
        let(:params) { { sortings: [{ field: 'checklist_name', value: 'desc' }] } }

        it 'sorts results' do
          expect(searcher.results.first.checklist.name)
            .to be > searcher.results.last.checklist.name
        end
      end
    end
  end
end
