# frozen_string_literal: true

require 'rails_helper'

describe PoliclinicSearcher do
  let_it_be(:searched) { create(:policlinic, size: 's', kind: 'children', time_to_inspect_minutes: 20) }
  let_it_be(:unrelated) { create_list(:policlinic, 2, size: 'xl', kind: 'adult', time_to_inspect_minutes: 100) }

  let_it_be(:default_scoped) { [searched, *unrelated] }

  let_it_be(:archived) { create(:policlinic, archived_at: Time.zone.now) }

  before_all { Policlinic.reindex }

  it_behaves_like 'Searchable', fields: %i[name], is_default_scoped: true

  describe 'base scope' do
    subject(:searcher) { described_class.call }

    it 'works' do
      expect(searcher.results).to include(searched)
      expect(searcher.results).not_to include(archived)
    end
  end

  describe 'filtering' do
    context 'by administrative_district_id' do
      subject(:searcher) do
        described_class.call(
          { filters: [{ field: 'administrative_district_id', value: [searched.administrative_district.id] }] }
        )
      end

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq searched
      end
    end

    context 'by district_id' do
      subject(:searcher) do
        described_class.call(
          { filters: [{ field: 'district_id', value: [searched.district_id] }] }
        )
      end

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq searched
      end
    end

    context 'by kind' do
      subject(:searcher) do
        described_class.call(
          { filters: [{ field: 'kind', value: [searched.kind] }] }
        )
      end

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq searched
      end
    end

    context 'by size' do
      subject(:searcher) do
        described_class.call(
          { filters: [{ field: 'size', value: [searched.size] }] }
        )
      end

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq searched
      end
    end
  end

  describe 'sorting' do
    it_behaves_like 'Sortable',
                    fields: %i[id name district.name administrative_district.name address kind size time_to_inspect_minutes]
  end
end
