# frozen_string_literal: true

require 'rails_helper'

describe UserSearcher do
  let_it_be(:current_project_type) { 'mfc' }
  let_it_be(:user) { create(:user, :superadmin) }
  let_it_be(:searched) { create(:user, :curator, project_type: 'mfc') }
  let_it_be(:curated) { create(:user, curator: searched, project_type: 'mfc') }
  let_it_be(:curated2) { create(:user, :inspector, :with_curator, project_type: 'mfc') }
  let_it_be(:unrelated) { create_list(:user, 2, :inspector, project_type: 'policlinic') }
  let_it_be(:archived_user) { create(:user, :inspector, :mfc, state: :archived) }
  let_it_be(:default_scoped) { [searched] }
  let_it_be(:admin) { create(:user, :admin, project_type: 'mfc') }
  let_it_be(:curator) { create(:user, :curator, project_type: 'mfc') }
  let_it_be(:inspector) { create(:user, curator: curator) }

  before_all { User.reindex }

  subject(:searcher) { described_class.call(params, user, current_project_type) }

  let(:params) { {} }

  it_behaves_like 'Searchable', fields: %i[full_name],
                                is_user_required: true,
                                is_default_scoped: true,
                                project_type_required: true,
                                class_name: User

  describe 'base scope' do
    it 'works' do
      expect(searcher.results).to include(searched)
      expect(searcher.results).not_to include(archived_user)
    end
  end

  describe 'after filter' do
    context 'as an admin of project' do
      let(:user) { admin }

      it 'does not show unrelated project users' do
        expect(searcher).not_to include(unrelated)
      end
    end

    context 'as curator' do
      let(:user) { curator }

      it 'only shows curated users' do
        expect(searcher.size).to eq 1
        expect(searcher.results).to include(inspector)
      end
    end
  end

  describe 'filtering' do
    context 'by role' do
      let(:params) { { filters: [{ field: 'role', value: ['curator'] }] } }

      it 'works' do
        expect(searcher.size).to eq 2
        expect(searcher.results).to include(searched, curator)
      end
    end

    context 'by curator_id' do
      let(:params) { { filters: [{ field: 'curator_id', value: [curated.curator_id.to_s] }] } }

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq curated
      end
    end

    context 'by state' do
      let(:params) { { filters: [{ field: 'state', value: ['archived'] }] } }

      it 'works' do
        expect(searcher.size).to eq 1
        expect(searcher.results.first).to eq archived_user
      end
    end
  end

  describe 'sorting' do
    it_behaves_like 'Sortable', fields: %i[id full_name role email last_sign_in_at],
                                is_user_required: true,
                                project_type_required: true

    context 'sorts by curator.full_name' do
      context 'asc' do
        subject(:searcher) { described_class.call(params, user, current_project_type) }

        let(:params) { { sortings: [{ field: 'curator.full_name', value: 'asc' }] } }

        it 'works' do
          expect(searcher.results[0].curator.full_name).to be < searcher.results[1].curator.full_name
        end
      end

      context 'desc' do
        subject(:searcher) { described_class.call(params, user, current_project_type) }

        let(:params) { { sortings: [{ field: 'curator.full_name', value: 'desc' }] } }

        it 'works' do
          expect(searcher.results[0].curator.full_name).to be > searcher.results[1].curator.full_name
        end
      end
    end
  end
end
