# frozen_string_literal: true

require 'rails_helper'

# experiment
RSpec::Matchers.define :only_return do |searched|
  match do |searcher|
    searcher.size == 1 && searcher.results.first == searched
  end

  # failure message is alright, but the documentation format message is ultra-fucked
  failure_message do |searcher|
    "expected searcher to return 1 result but got #{searcher.results.size}; expected #{searcher.results} to include #{searched.inspect}"
  end
end

describe VacationSearcher do
  let_it_be(:current_project_type) { 'mfc' }
  let_it_be(:searched) { create(:vacation, user: create(:user, :inspector, project_type: 'mfc')) }
  let_it_be(:unrelated) do
    create(:vacation,
           user: create(:user, :inspector, project_type: 'policlinic'),
           starts_at: 1.year.from_now,
           ends_at: 2.years.from_now)
  end
  let_it_be(:user) { create(:user, :superadmin) }
  let_it_be(:admin) { create(:user, :admin, project_type: 'mfc') }

  before_all { Vacation.reindex }

  subject(:searcher) { described_class.call(params, user, current_project_type) }

  describe 'searching' do
    context 'by user.full_name' do
      let(:params) { { search: searched.user.full_name } }

      it { is_expected.to only_return(searched) }
    end
  end

  describe 'filtering' do
    context 'by user.curator_id' do
      let(:params) { { filters: [{ field: 'user.curator_id', value: [searched.user.curator_id] }] } }

      it { is_expected.to only_return(searched) }
    end

    context 'by starts_at' do
      let(:params) { { filters: [{ field: 'starts_at', value: [searched.starts_at] }] } }

      it { is_expected.to only_return(searched) }
    end

    context 'by ends_at' do
      let(:params) { { filters: [{ field: 'ends_at', value: [searched.ends_at] }] } }

      it { is_expected.to only_return(searched) }
    end
  end

  describe 'after filter' do
    let(:params) { {} }

    context 'as an admin of project' do
      let(:user) { admin }

      it 'does not show unrelated project users vacations' do
        expect(searcher.results).to include(searched)
        expect(searcher.results).not_to include(unrelated)
      end
    end

    context 'as curator' do
      let(:user) { create(:user, :curator, :mfc) }
      let(:curated_user) { create(:user, :inspector, :mfc, curator: user) }
      let!(:curated_user_vacation) { create(:vacation, user: curated_user) }

      before { Vacation.reindex }

      it 'only shows curated users vacations' do
        expect(searcher.size).to eq 1
        expect(searcher.results).to include(curated_user_vacation)
      end
    end
  end

  describe 'sorting' do
    let_it_be(:sorted) do
      create(:vacation, user: create(:user, :inspector, project_type: 'mfc'), starts_at: 5.days.from_now, ends_at: 10.days.from_now)
    end

    before { Vacation.reindex }

    it_behaves_like 'Sortable', fields: %i[id starts_at ends_at user.full_name user.curator.full_name],
                                is_user_required: true,
                                project_type_required: true
  end
end
