import lodashDebounce from 'lodash.debounce'
import { db } from '~/db/index'
import { dateForRequest } from '~/assets/utils/helpers/index'
import { SYNC } from '~/assets/utils/constants'

let loaderTimerId = null
const DELAY_BEFORE_LOADER_WILL_SHOW = 700

const saveInspectionToServer = lodashDebounce(async function(inspection) {
  await this.$inspections.update(inspection.id, {
    state: inspection.state,
    rawData: inspection.rawData
  })
}, 2000)

export const state = () => ({
  inspections: [],
  currentInspection: null,
  selectedDate: dateForRequest(new Date()),
  networkError: null,
  isLoading: false,
  isLoaderShow: false
})

export const getters = {
  getAll: state => state.inspections,
  get: state => state.currentInspection,
  networkError: state => state.networkError,
  selectedDate: state => state.selectedDate,
  isLoading: state => state.isLoading,
  isLoaderShow: state => state.isLoaderShow
}

export const mutations = {
  SET_INSPECTIONS(state, value) {
    state.inspections = value
  },
  SET_CURRENT_INSPECTION(state, value) {
    state.currentInspection = value
  },
  SET_NETWORK_ERROR(state, value) {
    state.networkError = value
  },
  SET_DATE(state, value) {
    state.selectedDate = value
  },
  CLEAR_INSPECTIONS(state) {
    state.inspections = []
  },
  CLEAR_CURRENT_INSPECTION(state) {
    state.currentInspection = null
  },
  SET_LOADING_STATE(state, value) {
    state.isLoading = value
  },
  SET_LOADER_STATE(state, value) {
    state.isLoaderShow = value
  }
}

export const actions = {
  setDate({ commit }, date) {
    commit('SET_DATE', date)
  },
  loadInspection({ commit, getters, rootGetters }, id) { // show page
    commit('CLEAR_CURRENT_INSPECTION')
    commit('SET_NETWORK_ERROR', null)

    commit('SET_LOADING_STATE', true)
    loaderTimerId = setTimeout(() => {
      commit('SET_LOADER_STATE', true)
    }, DELAY_BEFORE_LOADER_WILL_SHOW)

    this.$load(async() => {
      let inspection = getters.getAll.find(el => el.id === id)
      if (!inspection) {
        const inspectionDB = await db.inspections.get(id)
        inspection = inspectionDB ? inspectionDB.inspection : null
      }
      if (!inspection) {
        inspection = await this.$inspections.show(id)
        inspection = {
          ...inspection,
          sync: SYNC.ALREADY_SYNC,
          lastSyncDate: new Date()
        }
      }
      commit('SET_CURRENT_INSPECTION', inspection || null)
      clearTimeout(loaderTimerId)
      commit('SET_LOADING_STATE', false)
      commit('SET_LOADER_STATE', false)
    }, (e) => { // cannot get inspections from api or cache
      console.error(e)
      if (!rootGetters['connection/isOnline']) {
        commit('SET_NETWORK_ERROR', e)
      }
      clearTimeout(loaderTimerId)
      commit('SET_LOADING_STATE', false)
      commit('SET_LOADER_STATE', false)
    })
  },

  loadInspections({ commit, dispatch, rootGetters }, date) { // index page
    commit('CLEAR_INSPECTIONS')
    commit('SET_NETWORK_ERROR', null)

    commit('SET_LOADING_STATE', true)
    loaderTimerId = setTimeout(() => {
      commit('SET_LOADER_STATE', true)
    }, DELAY_BEFORE_LOADER_WILL_SHOW)

    const isOnline = rootGetters['connection/isOnline']

    this.$load(async() => {
      const isToday = dateForRequest(new Date()) === date
      let inspections = []

      if (isToday) {
        if (isOnline) {
          await dispatch('syncWithServer')
        } else {
          inspections = await db.inspections.where({ date }).toArray()
        }
      }

      if (!inspections.length) {
        inspections = (await this.$inspections.index({ date })).inspections
        if (isToday) {
          await db.inspections.clear()
          await db.inspections.bulkAdd(inspections.map((el) => {
            return {
              id: el.id,
              date,
              sync: SYNC.ALREADY_SYNC,
              lastSyncDate: new Date(),
              inspection: {
                ...el,
                sync: SYNC.ALREADY_SYNC,
                lastSyncDate: new Date()
              }
            }
          }))
        }
      } else {
        inspections = inspections.map(el => el.inspection)
      }

      inspections.forEach((inspection) => {
        const checklist = JSON.parse(inspection.rawData)
        checklist.files && checklist.files.forEach((file) => {
          dispatch('files/downloadFile', file, { root: true })
        })
      })

      commit('SET_INSPECTIONS', inspections.map((el) => {
        return {
          ...el,
          sync: SYNC.ALREADY_SYNC,
          lastSyncDate: new Date()
        }
      }))
      clearTimeout(loaderTimerId)
      commit('SET_LOADING_STATE', false)
      commit('SET_LOADER_STATE', false)
    }, (e) => { // cannot get inspections from api or cache
      console.error(e)
      if (!rootGetters['connection/isOnline']) {
        commit('SET_NETWORK_ERROR', e)
      }
      clearTimeout(loaderTimerId)
      commit('SET_LOADING_STATE', false)
      commit('SET_LOADER_STATE', false)
    })
  },

  saveInspection({ commit, getters, rootGetters }, inspection) {
    this.$load(async() => {
      const date = dateForRequest(inspection.startsAt)
      const isOnline = rootGetters['connection/isOnline']

      if (isOnline) {
        this.$load(async() => {
          inspection = {
            ...inspection,
            sync: SYNC.ALREADY_SYNC,
            lastSyncDate: new Date()
          }

          if (getters.get.id === inspection.id) {
            commit('SET_CURRENT_INSPECTION', inspection)
          }
          commit('SET_INSPECTIONS', getters.getAll.map(el => el.id === inspection.id ? inspection : el))

          await saveInspectionToServer.call(this, inspection)

          await db.inspections.put({
            id: inspection.id,
            date,
            inspection,
            sync: SYNC.ALREADY_SYNC,
            lastSyncDate: new Date()
          })
        })
      } else {
        inspection = {
          ...inspection,
          sync: SYNC.NEED_SYNC
        }

        if (getters.get.id === inspection.id) {
          commit('SET_CURRENT_INSPECTION', inspection)
        }
        commit('SET_INSPECTIONS', getters.getAll.map(el => el.id === inspection.id ? inspection : el))

        await db.inspections.update(inspection.id, {
          date,
          inspection,
          sync: SYNC.NEED_SYNC
        })
      }
    })
  },

  async syncWithServer({ commit, getters }) {
    const today = dateForRequest(new Date())
    const inspections = await db.inspections.where({ date: today, sync: SYNC.NEED_SYNC }).toArray()

    const promises = inspections.map(({ inspection }) => {
      return this.$inspections.update(inspection.id, {
        state: inspection.state,
        rawData: inspection.rawData
      })
    })

    const results = await Promise.allSettled(promises)
    results.forEach(async({ status, value }) => {
      if (status === 'fulfilled') {
        const inspection = {
          ...value,
          sync: SYNC.ALREADY_SYNC,
          lastSyncDate: new Date()
        }
        if (getters.get.id === inspection.id) {
          commit('SET_CURRENT_INSPECTION', inspection)
        }
        commit('SET_INSPECTIONS', getters.getAll.map(el => el.id === inspection.id ? inspection : el))
        await db.inspections.update(value.id, {
          sync: SYNC.ALREADY_SYNC,
          lastSyncDate: new Date(),
          inspection
        })
      }
    })
  }
}
