import { Vue, Component, Watch } from 'nuxt-property-decorator'
import { mapState } from 'vuex'
import {
  ISortItem, ICommonIndexPayload, ISortDirection, ICommonIndexPayloadURL,
  ICommonIndexPayloadForBeckEnd, IFilterItemForBeckEnd
} from '~/ts/interfaces'
import { payloadForURL } from '~/assets/utils/helpers'

@Component({
  computed: {
    ...mapState({
      items: 'indexPageItems',
      pageSettings: 'indexPageSettings',
      indexPage: 'indexPage'
    })
  }
})
export default class TTableMixin extends Vue {
  indexPage: string
  pageSettings: any
  items: any
  options: {
    page: number
    pageCount: number
    itemsLength: number
    perPage: number
  } = {
    page: 0,
    pageCount: 0,
    itemsLength: 0,
    perPage: 10
  }

  mobileBreakpoint = 1025

  initialPayload: any

  payload: ICommonIndexPayload = {
    perPage: 10,
    page: 0
  }

  footerProps: {
    itemsPerPageText: string
    itemsPerPageAllText: string
    pageText: string
  } = {
    itemsPerPageText: 'Количество строк:',
    itemsPerPageAllText: 'Все',
    pageText: ''
  }

  debounceSearch: any = null
  isLoading: boolean = false

  sortBy: string[] = []
  sortDesc: boolean[] = []
  oldProjectTypeValue: string

  created() {
    this.oldProjectTypeValue = this.$auth.user.projectType
    if (this.$route.query) {
      const queryPayload = this.$clone(this.$route.query)
      this.payload = {
        ...this.payload,
        ...this.getPayloadFromURL(queryPayload as ICommonIndexPayloadURL)
      }
      this.options.perPage = this.payload.perPage > 15 ? -1 : this.payload.perPage
    }
    if (this.indexPage !== this.$route.name) {
      this.getList()
    } else {
      this.setPageSettings()
    }
  }

  getList() {}

  handleTableFilterChange() {
    this.payload.page = 1
    this.getList()
  }

  changePage(page: number) {
    if (page !== this.payload.page) {
      this.payload.page = page
      this.getList()
    }
  }

  setPageSettings() {
    this.options.page = this.pageSettings.currentPage
    this.options.pageCount = this.pageSettings.totalPages
    this.options.itemsLength = this.pageSettings.totalCount
  }

  changePerPage(perPage: number) {
    this.options.perPage = perPage
    this.payload.perPage = perPage === -1 ? this.options.itemsLength : perPage
    if (this.payload.perPage) {
      this.getList()
    }
  }

  // не сортируем на фронте
  customSort(items: any[]) {
    return items
  }

  get payloadWithValues() : ICommonIndexPayloadForBeckEnd {
    const payload = this.$clone(this.payload)
    const newFilters: IFilterItemForBeckEnd[] = []
    const newSorting: ISortItem[] = []
    for (const key in payload.filters) {
      const value = payload.filters[key].value
      const range = payload.filters[key].range
      if (value && value[0]) {
        newFilters.push({
          field: payload.filters[key].field,
          value: payload.filters[key].value
        })
      } else if (range && (range.gte || range.lte)) {
        newFilters.push({
          field: payload.filters[key].field,
          range: {
            gte: range.gte,
            lte: range.lte
          }
        })
      }
    }

    payload.sortings?.forEach((sortEl: ISortItem) => {
      const isSortAlreadyExists = newSorting.find((el: ISortItem) => el.field === sortEl.field)
      if (sortEl.value && !isSortAlreadyExists) {
        newSorting.push(sortEl)
      }
    })

    const newPayload: ICommonIndexPayloadForBeckEnd = {
      ...payload,
      ...{ filters: newFilters },
      ...{ sortings: newSorting }
    }
    return newPayload
  }

  getPayloadFromURL(payload: ICommonIndexPayloadURL) : ICommonIndexPayload {
    const newPayload: ICommonIndexPayload = this.$clone(this.payload)
    newPayload.perPage = Number(payload.perPage) || 10
    newPayload.page = Number(payload.page) || 1
    newPayload.search = payload.search

    let sortBy : string = ''
    let sortDirection : ISortDirection = ''
    for (const key in payload) {
      if (this.isFilterKey(key)) {
        let formatKey = ''
        let isRange = false
        let isStart = false
        if (key.includes('-')) {
          const arrKey = key.split('-')
          formatKey = arrKey[0]
          isStart = arrKey[1] === 'start'
          isRange = true
        }
        for (const filterKey in newPayload.filters) {
          if (newPayload.filters[filterKey].field === key || newPayload.filters[filterKey].field === formatKey) {
            if (isRange) {
              if (isStart) {
                newPayload.filters[filterKey].range = {
                  gte: payload[key],
                  lte: newPayload.filters[filterKey].range?.lte || ''
                }
              } else {
                newPayload.filters[filterKey].range = {
                  gte: newPayload.filters[filterKey].range?.gte || '',
                  lte: payload[key]
                }
              }
            } else if (payload[key]) {
              if (Number(payload[key])) {
                newPayload.filters[filterKey].value = [Number(payload[key])]
              } else {
                newPayload.filters[filterKey].value = [payload[key]]
              }
            }
          }
        }
      } else if (key === 'sortBy') {
        sortBy = payload[key] as string
      } else if (key === 'sortDesc') {
        sortDirection = payload[key] === 'true' ? 'desc' : 'asc'
      }
    }

    if (newPayload.sortings && sortBy) {
      newPayload.sortings.push(
        { field: sortBy, value: sortDirection }
      )
      this.sortBy.push(sortBy)
      this.sortDesc.push(sortDirection === 'desc')
    }
    return newPayload
  }

  isFilterKey(key: string) : boolean {
    const keys = [
      'perPage',
      'page',
      'search',
      'sortBy',
      'sortDesc'
    ]
    return !keys.includes(key)
  }

  isEqual(firstArray: boolean[], secondArray: boolean[]): boolean {
    return firstArray.length === secondArray.length && firstArray.every((el, i) => el === secondArray[i])
  }

  isItemsLengthChenged(perPage: number): number {
    if (
      perPage * this.options.page > this.options.itemsLength ||
      this.items.length > perPage * this.options.page
    ) {
      return this.items.length
    } else {
      return perPage * this.options.page
    }
  }

  @Watch('payload', { deep: true })
  savePayloadToQueryParams() {
    this.$router.replace({ query: payloadForURL(this.payload) }).catch(() => {})
  }

  @Watch('pageSettings', { deep: true })
  setNewPageSettings() {
    this.setPageSettings()
  }

  @Watch('options', { deep: true })
  setFooterProps() {
    const perPage = this.options.perPage === -1 ? this.options.itemsLength : this.options.perPage
    const lastElement = this.isItemsLengthChenged(perPage)
    const firstElement = (perPage * (this.options.page - 1)) + 1
    const intervalItem = `${firstElement} - ${lastElement}`
    this.footerProps.pageText = `${intervalItem} из ${this.options.itemsLength}`
  }

  @Watch('sortDesc', { deep: true })
  acceptSorting(oldValue: boolean[], newValue: boolean[]) {
    if (this.payload.sortings && !this.isEqual(oldValue, newValue)) {
      this.payload.sortings = this.payload.sortings.map((el: ISortItem) => {
        const sortIndex = this.sortBy.indexOf(el.field)
        let sortDirection : ISortDirection = ''
        if (sortIndex !== -1) {
          sortDirection = this.sortDesc[sortIndex] ? 'desc' : 'asc'
        }
        return {
          field: el.field,
          value: sortDirection
        }
      })
      this.getList()
    }
  }

  @Watch('$auth.user.projectType')
  onProjectTypeChange(newProjectTypeValue: string) {
    if (this.oldProjectTypeValue === newProjectTypeValue) {
      return
    }
    this.oldProjectTypeValue = newProjectTypeValue
    this.payload = this.$clone(this.initialPayload)

    if (this.$auth.user.isSuperAdmin && this.payload?.filters?.projectType) {
      this.payload.filters.projectType.value = [newProjectTypeValue]
    }

    if (this.indexPage !== 'policlinics' && this.indexPage !== 'mfcs') {
      this.getList()
    }
  }
}
