import SparkMD5 from 'spark-md5'

export const getFileMetadata = (file) => {
  return new Promise((resolve, reject) => {
    const instanse = new FileChecksum(file)
    instanse.startRead((error, checksum) => {
      if (error) {
        reject(error)
        return
      }
      resolve({
        checksum,
        filename: file.name,
        contentType: file.type,
        byteSize: file.size
      })
    })
  })
}

export const directUpload = (url, headers, file) => {
  const upload = new BlobUpload({ file, directUploadData: { url, headers } })
  return new Promise((resolve, reject) => {
    upload.create((error) => {
      if (error) {
        reject(error)
      } else {
        resolve()
      }
    })
  })
}

class BlobUpload {
  constructor(blob) {
    this.blob = blob
    this.file = blob.file

    const { url, headers } = blob.directUploadData

    this.xhr = new XMLHttpRequest()
    this.xhr.open('PUT', url, true)
    this.xhr.responseType = 'text'
    for (const key in headers) {
      const kebabCase = key.replace(/([a-z0-9])([A-Z])/g, '$1-$2')
      const headerKey = `${kebabCase.slice(0, 1).toUpperCase()}${kebabCase.slice(1)}`
      this.xhr.setRequestHeader(headerKey, headers[key])
    }
    this.xhr.addEventListener('load', event => this.requestDidLoad(event))
    this.xhr.addEventListener('error', event => this.requestDidError(event))
  }

  create(callback) {
    this.callback = callback
    this.xhr.send(this.file.slice())
  }

  requestDidLoad(event) {
    const { status, response } = this.xhr
    if (status >= 200 && status < 300) {
      this.callback(null, response)
    } else {
      this.requestDidError(event)
    }
  }

  requestDidError() {
    this.callback(
          `Error storing "${this.file.name}". Status: ${this.xhr.status}`
    )
  }
}

let fileSlice
if (process.browser) {
  fileSlice = File.prototype.slice || File.prototype.mozSlice || File.prototype.webkitSlice
}

class FileChecksum {
  constructor(file) {
    this.file = file
    this.chunkSize = 2097152 // 2MB
    this.chunkCount = Math.ceil(this.file.size / this.chunkSize)
    this.chunkIndex = 0
  }

  startRead(callback) {
    this.callback = callback
    this.md5Buffer = new SparkMD5.ArrayBuffer()
    this.fileReader = new FileReader()
    this.fileReader.addEventListener('load', event => this.fileReaderDidLoad(event))
    this.fileReader.addEventListener('error', event => this.fileReaderDidError(event))
    this.readNextChunk()
  }

  fileReaderDidLoad(event) {
    this.md5Buffer.append(event.target.result)
    if (!this.readNextChunk()) {
      const binaryDigest = this.md5Buffer.end(true)
      const base64digest = btoa(binaryDigest)
      this.callback(null, base64digest)
    }
  }

  fileReaderDidError() {
    this.callback(`Error reading ${this.file.name}`)
  }

  readNextChunk() {
    if (this.chunkIndex < this.chunkCount || (this.chunkIndex === 0 && this.chunkCount === 0)) {
      const start = this.chunkIndex * this.chunkSize
      const end = Math.min(start + this.chunkSize, this.file.size)
      const bytes = fileSlice.call(this.file, start, end)
      this.fileReader.readAsArrayBuffer(bytes)
      this.chunkIndex++
      return true
    } else {
      return false
    }
  }
}
