import { db } from '~/db/index'
import { getFileMetadata, directUpload } from '~/assets/utils/helpers'

import { FILE_STATES } from '~/assets/utils/constants'

const changeFileState = (file, newState) => {
  return {
    ...file,
    state: newState
  }
}

const changeFilesState = (files, newState) => {
  return files.map(file => changeFileState(file, newState))
}

const loadToIndexedDB = async(DBFile) => {
  try {
    await db.files.add(DBFile)
    return true
  } catch (e) {
    console.error(e)
    return false
  }
}

const updateFileInIndexedDB = async(DBFile) => {
  try {
    await db.files.put(DBFile)
  } catch (e) {
    console.error(e)
  }
}

const deleteFileFromIndexedDB = async(DBFile) => {
  try {
    await db.files.delete(DBFile.id)
  } catch (e) {
    console.error(e)
  }
}

async function getDataForUpload(DBFile) {
  try {
    const fileMetadata = await getFileMetadata(DBFile.value)
    return await this.$axios.post('/direct_uploads', fileMetadata)
  } catch (e) {
    console.error(e)
    return false
  }
}

const uploadS3 = async(DBFile) => {
  try {
    await directUpload(DBFile.url, DBFile.headers, DBFile.value)
    return true
  } catch (e) {
    console.error(e)
    return false
  }
}

async function attachFile(DBFile) {
  try {
    await this.$axios.post(`/inspections/${DBFile.inspectionId}/attach_file`, {
      signedId: DBFile.signedId
    })
    return true
  } catch (e) {
    console.error(e)
    return false
  }
}

async function purgeFile(DBFile) {
  try {
    await this.$axios.$delete(`/inspections/${DBFile.inspectionId}/purge_file`, {
      params: {
        key: DBFile.key
      }
    })
    return true
  } catch (e) {
    console.error(e)
    const code = e.response?.data?.code
    const status = e.response?.status
    if (code === 404 || status === 422) {
      return true
    }
    return false
  }
}

async function downloadFile(file) {
  try {
    const { url } = (await this.$axios(file.S3href)).data
    const instance = this.$axios.create()
    delete instance.defaults.headers.common.Authorization
    const response = await instance({
      url,
      baseURL: '',
      responseType: 'blob'
    })
    return {
      ...file,
      value: response.data
    }
  } catch (e) {
    console.error(e)
  }
}

export const state = () => ({
  files: [],
  currentInspectionId: 0
})

export const getters = {
  get: (state, _getters, _rootState) => {
    return state.files.filter((file) => {
      return file.inspectionId === state.currentInspectionId
    })
  },
  getAll: state => state.files,
  getNotLoadedFiles: state => state.files.filter((el) => {
    return [
      FILE_STATES.INITIAL,
      FILE_STATES.UPLOADED_TO_INDEXED_DB,
      FILE_STATES.DIRECT_UPLOAD_COMPLETE,
      FILE_STATES.S3_UPLOAD_COMPLETE
    ].includes(el.state)
  }),
  getCurrentInspectionId: state => state.currentInspectionId
}

export const mutations = {
  SET_FILES(state, newFiles) {
    state.files = newFiles
  },
  UPDATE_FILE(state, newFile) {
    state.files = state.files.map(file => file.id === newFile.id ? newFile : file)
  },
  ADD_FILE(state, newFile) {
    state.files.push(newFile)
  },
  DELETE_FILE(state, file) {
    state.files = state.files.filter(f => f.id !== file.id)
  },
  SET_CURRET_INSPECTIONS_ID(state, id) {
    state.currentInspectionId = id
  }
}

export const actions = {
  setCurrentInspectionId({ commit }, id) {
    commit('SET_CURRET_INSPECTIONS_ID', id)
  },
  continueUploadFiles({ dispatch, getters }) {
    const DBFiles = getters.getAll
    dispatch('uploadFiles', DBFiles)
  },
  async syncStoreWithIndexedDb({ commit, dispatch }) {
    const DBFiles = await db.files.toArray()
    commit('SET_FILES', DBFiles)
    dispatch('continueUploadFiles')
  },
  deleteFile({ dispatch, commit }, DBFile) {
    DBFile = changeFileState(DBFile, FILE_STATES.READY_TO_DELETE)
    updateFileInIndexedDB(DBFile)
    commit('UPDATE_FILE', DBFile)
    dispatch('loadFile', DBFile)
  },
  addNewFiles({ dispatch }, DBFiles) {
    DBFiles = changeFilesState(DBFiles, FILE_STATES.INITIAL)
    dispatch('uploadFiles', DBFiles)
  },
  uploadFiles({ dispatch }, DBFiles) {
    DBFiles.forEach((file) => {
      if (file.state !== FILE_STATES.ATTACH_FILE_COMPLETE) {
        dispatch('loadFile', file)
      }
    })
  },
  async loadFile({ commit }, DBFile) {
    if (DBFile.state === FILE_STATES.INITIAL) {
      if (await loadToIndexedDB(DBFile)) {
        DBFile = changeFileState(DBFile, FILE_STATES.UPLOADED_TO_INDEXED_DB)
        updateFileInIndexedDB(DBFile)
        commit('ADD_FILE', DBFile)
      }
    }

    if (DBFile.state === FILE_STATES.UPLOADED_TO_INDEXED_DB) {
      const result = await getDataForUpload.call(this, DBFile)
      if (result) {
        const { data } = result
        const { url, headers } = data.directUpload
        const s3href = `${process.env.IMG_URL}/rails/active_storage/blobs/${data.signedId}/${data.filename}`
        DBFile = changeFileState(DBFile, FILE_STATES.DIRECT_UPLOAD_COMPLETE)
        DBFile = {
          ...DBFile,
          signedId: data.signedId,
          key: data.key,
          S3href: s3href,
          url,
          headers,
          error: false
        }
      } else {
        DBFile = {
          ...DBFile,
          error: true
        }
      }

      updateFileInIndexedDB(DBFile)
      commit('UPDATE_FILE', DBFile)
    }

    if (DBFile.state === FILE_STATES.DIRECT_UPLOAD_COMPLETE) {
      if (await uploadS3(DBFile)) {
        DBFile = {
          ...changeFileState(DBFile, FILE_STATES.S3_UPLOAD_COMPLETE),
          error: false
        }
      } else {
        DBFile = {
          ...DBFile,
          error: true
        }
      }
      updateFileInIndexedDB(DBFile)
      commit('UPDATE_FILE', DBFile)
    }

    if (DBFile.state === FILE_STATES.S3_UPLOAD_COMPLETE) {
      if (await attachFile.call(this, DBFile)) {
        DBFile = {
          ...changeFileState(DBFile, FILE_STATES.ATTACH_FILE_COMPLETE),
          error: false
        }
      } else {
        DBFile = {
          ...DBFile,
          error: true
        }
      }
      updateFileInIndexedDB(DBFile)
      commit('UPDATE_FILE', DBFile)
    }

    if (DBFile.state === FILE_STATES.READY_TO_DELETE) {
      if (await purgeFile.call(this, DBFile)) {
        deleteFileFromIndexedDB(DBFile)
        commit('DELETE_FILE', DBFile)
      }
    }

    if (DBFile.state === FILE_STATES.NOT_DOWNLOADED) {
      let file = {
        ...await downloadFile.call(this, DBFile),
        error: false
      }
      if (file) {
        file = changeFileState(file, FILE_STATES.DOWNLOADED)
        updateFileInIndexedDB(file)
        commit('UPDATE_FILE', file)
      }
    }
  },
  repeatFileUpload({ getters, dispatch }, fileId) {
    const file = getters.getAll.find(el => el.id === fileId)
    if (file) {
      dispatch('loadFile', file)
    }
  },
  async downloadFile({ commit, getters, dispatch }, file) {
    if (!getters.getAll.find(f => f.id === file.id) && file.S3href) {
      const fileWithSameKey = getters.getAll.find(f => f.key === file.key)
      if (!fileWithSameKey) {
        file = changeFileState(file, FILE_STATES.NOT_DOWNLOADED)
        updateFileInIndexedDB(file)
        commit('ADD_FILE', file)
        await dispatch('loadFile', file)
      } else {
        await dispatch('loadFile', fileWithSameKey)
      }
    }
  }
}
